/*
*  Name:      application.cc
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   UESQLC application
*  Date:      $Date: 1999/01/01 18:08:48 $
*  Revision:  $Revision: 1.9 $
*
*  Copyright (C) 1996-1998  Rafael Jesus Alcantara Perez <ralcan@vnet.es>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#include <cstdio>
#include <cstring>
#include <fstream>
#include <mpcl/operating_system.hh>
#include "../../config.h"
#include "defs.hh"
#include "stack_value.hh"
#include "uesql_parser.h"
#include "application.hh"


/*
*  G L O B A L   T E M P L A T E   E X P L I C I T   I N S T A N T I A T I O N S
*/

template class TCollection<long int>;
template class TCollection<TSmartPointer <TPatternBasedCodeGenerator> >;
template class TCollection<TVariableMap>;
template class TBaseContainer<long int>;
template class TBaseContainer<TSmartPointer <TPatternBasedCodeGenerator> >;
template class TBaseContainer<TVariableMap>;
template class TLinkedContainer<long int>;
template class TLinkedContainer<TSmartPointer <TPatternBasedCodeGenerator> >;
template class TLinkedContainer<TVariableMap>;
template class TBaseMap<TString, long int>;
template class TBaseMap<TString, TSmartPointer <TPatternBasedCodeGenerator> >;
template class TBaseMap<TString, TVariableMap>;
template class TLinkedMap<TString, long int>;
template class TLinkedMap<TString, TSmartPointer <TPatternBasedCodeGenerator> >;
template class TLinkedMap<TString, TVariableMap>;
template class TMap<TString, long int>;
template class TSmartPointer<TPatternBasedCodeGenerator>;


/*
*  C O N S T R U C T O R S
*/

TApplication::TApplication ( const char* pkcAPPLICATION_NAME    ,
                             const char* pkcAPPLICATION_RELEASE )                :
  TBaseApplication                 (pkcAPPLICATION_NAME, pkcAPPLICATION_RELEASE) ,
  tRequirementVerifier             ()                                            ,
  tConfigProcessor                 ()                                            ,
  uiCfgDebug                       (0)                                           ,
  gCfgHelp                         (false)                                       ,
  gCfgVersion                      (false)                                       ,
  gCfgParameters                   (false)                                       ,
  gCfgOutputCdml                   (false)                                       ,
  gCfgOutputCdmlSkel               (false)                                       ,
  gCfgLineDirectives               (false)                                       ,
  stCfgInputFileName               ()                                            ,
  stCfgOutputFileName              ()                                            ,
  stCfgCdmlPath                    ()                                            ,
  stCfgTarget                      ()                                            ,
  gCompilationOk                   (false)                                       ,
  gReusingTarget                   (false)                                       ,
  zEmbeddedIntroducerLine          (0)                                           ,
  tReservedWordMap                 ()                                            ,
  tTargetToCodeGeneratorMap        ()                                            ,
  tTargetToRuntimeParametersMapMap ()                                            ,
  tCodeParametersMap               ()                                            ,
  ptRuntimeParametersMap           (NULL)                                        ,
  stOutputCode                     ()                                            ,
  stPrefixOutputCode               ()                                            ,
  stPostfixOutputCode              ()                                            ,
  stCurrentTarget                  ()                                            ,
  stLastToken                      ()
{

  /*
  *  Subsection 5.2: <token> and <separator>. 
  */
	
  /*
  *  No-Terminal:  <reserved word>
  */
  tReservedWordMap.bind ("ABSOLUTE",          RW_ABSOLUTE);
  tReservedWordMap.bind ("ACTION",            RW_ACTION);
  tReservedWordMap.bind ("ADD",               RW_ADD);
  tReservedWordMap.bind ("ALLOCATE",          RW_ALLOCATE);
  tReservedWordMap.bind ("ALL",               RW_ALL);
  tReservedWordMap.bind ("ALTER",             RW_ALTER);
  tReservedWordMap.bind ("AND",               RW_AND);
  tReservedWordMap.bind ("ANY",               RW_ANY);
  tReservedWordMap.bind ("ASC",               RW_ASC);
  tReservedWordMap.bind ("ASSERTION",         RW_ASSERTION);
  tReservedWordMap.bind ("AS",                RW_AS);
  tReservedWordMap.bind ("AT",                RW_AT);
  tReservedWordMap.bind ("AUTHORIZATION",     RW_AUTHORIZATION);
  tReservedWordMap.bind ("AVG",               RW_AVG);
  tReservedWordMap.bind ("BETWEEN",           RW_BETWEEN);
  tReservedWordMap.bind ("BIT_LENGTH",        RW_BIT_LENGTH);
  tReservedWordMap.bind ("BIT",               RW_BIT);
  tReservedWordMap.bind ("BOTH",              RW_BOTH);
  tReservedWordMap.bind ("BY",                RW_BY);
  tReservedWordMap.bind ("CASCADED",          RW_CASCADED);
  tReservedWordMap.bind ("CASCADE",           RW_CASCADE);
  tReservedWordMap.bind ("CASE",              RW_CASE);
  tReservedWordMap.bind ("CAST",              RW_CAST);
  tReservedWordMap.bind ("CATALOG",           RW_CATALOG);
  tReservedWordMap.bind ("CHARACTER_LENGTH",  RW_CHARACTER_LENGTH);
  tReservedWordMap.bind ("CHARACTER",         RW_CHARACTER);
  tReservedWordMap.bind ("CHAR_LENGTH",       RW_CHAR_LENGTH);
  tReservedWordMap.bind ("CHAR",              RW_CHAR);
  tReservedWordMap.bind ("CHECK",             RW_CHECK);
  tReservedWordMap.bind ("CLOSE",             RW_CLOSE);
  tReservedWordMap.bind ("COALESCE",          RW_COALESCE);
  tReservedWordMap.bind ("COLLATE",           RW_COLLATE);
  tReservedWordMap.bind ("COLLATION",         RW_COLLATION);
  tReservedWordMap.bind ("COLUMN",            RW_COLUMN);
  tReservedWordMap.bind ("COMMIT",            RW_COMMIT);
  tReservedWordMap.bind ("CONNECTION",        RW_CONNECTION);
  tReservedWordMap.bind ("CONNECT",           RW_CONNECT);
  tReservedWordMap.bind ("CONSTRAINTS",       RW_CONSTRAINTS);
  tReservedWordMap.bind ("CONSTRAINT",        RW_CONSTRAINT);
  tReservedWordMap.bind ("CONTINUE",          RW_CONTINUE);
  tReservedWordMap.bind ("CONVERT",           RW_CONVERT);
  tReservedWordMap.bind ("CORRESPONDING",     RW_CORRESPONDING);
  tReservedWordMap.bind ("COUNT",             RW_COUNT);
  tReservedWordMap.bind ("CREATE",            RW_CREATE);
  tReservedWordMap.bind ("CROSS",             RW_CROSS);
  tReservedWordMap.bind ("CURRENT_DATE",      RW_CURRENT_DATE);
  tReservedWordMap.bind ("CURRENT_TIMESTAMP", RW_CURRENT_TIMESTAMP);
  tReservedWordMap.bind ("CURRENT_TIME",      RW_CURRENT_TIME);
  tReservedWordMap.bind ("CURRENT_USER",      RW_CURRENT_USER);
  tReservedWordMap.bind ("CURRENT",           RW_CURRENT);
  tReservedWordMap.bind ("CURSOR",            RW_CURSOR);
  tReservedWordMap.bind ("DATE",              RW_DATE);
  tReservedWordMap.bind ("DAY",               RW_DAY);
  tReservedWordMap.bind ("DEALLOCATE",        RW_DEALLOCATE);
  tReservedWordMap.bind ("DECIMAL",           RW_DECIMAL);
  tReservedWordMap.bind ("DECLARE",           RW_DECLARE);
  tReservedWordMap.bind ("DEC",               RW_DEC);
  tReservedWordMap.bind ("DEFAULT",           RW_DEFAULT);
  tReservedWordMap.bind ("DEFERRABLE",        RW_DEFERRABLE);
  tReservedWordMap.bind ("DEFERRED",          RW_DEFERRED);
  tReservedWordMap.bind ("DELETE",            RW_DELETE);
  tReservedWordMap.bind ("DESCRIBE",          RW_DESCRIBE);
  tReservedWordMap.bind ("DESCRIPTOR",        RW_DESCRIPTOR);
  tReservedWordMap.bind ("DESC",              RW_DESC);
  tReservedWordMap.bind ("DIAGNOSTICS",       RW_DIAGNOSTICS);
  tReservedWordMap.bind ("DISCONNECT",        RW_DISCONNECT);
  tReservedWordMap.bind ("DISTINCT",          RW_DISTINCT);
  tReservedWordMap.bind ("DOMAIN",            RW_DOMAIN);
  tReservedWordMap.bind ("DOUBLE",            RW_DOUBLE);
  tReservedWordMap.bind ("DROP",              RW_DROP);
  tReservedWordMap.bind ("ELSE",              RW_ELSE);
  tReservedWordMap.bind ("END",               RW_END);
  tReservedWordMap.bind ("ESCAPE",            RW_ESCAPE);
  tReservedWordMap.bind ("EXCEPTION",         RW_EXCEPTION);
  tReservedWordMap.bind ("EXCEPT",            RW_EXCEPT);
  tReservedWordMap.bind ("EXECUTE",           RW_EXECUTE);
  tReservedWordMap.bind ("EXISTS",            RW_EXISTS);
  tReservedWordMap.bind ("EXTRACT",           RW_EXTRACT);
  tReservedWordMap.bind ("FALSE",             RW_FALSE);
  tReservedWordMap.bind ("FETCH",             RW_FETCH);
  tReservedWordMap.bind ("FIRST",             RW_FIRST);
  tReservedWordMap.bind ("FLOAT",             RW_FLOAT);
  tReservedWordMap.bind ("FOREIGN",           RW_FOREIGN);
  tReservedWordMap.bind ("FOR",               RW_FOR);
  tReservedWordMap.bind ("FOUND",             RW_FOUND);
  tReservedWordMap.bind ("FROM",              RW_FROM);
  tReservedWordMap.bind ("FULL",              RW_FULL);
  tReservedWordMap.bind ("GET",               RW_GET);
  tReservedWordMap.bind ("GLOBAL",            RW_GLOBAL);
  tReservedWordMap.bind ("GOTO",              RW_GOTO);
  tReservedWordMap.bind ("GO",                RW_GO);
  tReservedWordMap.bind ("GRANT",             RW_GRANT);
  tReservedWordMap.bind ("GROUP",             RW_GROUP);
  tReservedWordMap.bind ("HAVING",            RW_HAVING);
  tReservedWordMap.bind ("HOUR",              RW_HOUR);
  tReservedWordMap.bind ("IMMEDIATE",         RW_IMMEDIATE);
  tReservedWordMap.bind ("INDICATOR",         RW_INDICATOR);
  tReservedWordMap.bind ("INITIALLY",         RW_INITIALLY);
  tReservedWordMap.bind ("INNER",             RW_INNER);
  tReservedWordMap.bind ("INPUT",             RW_INPUT);
  tReservedWordMap.bind ("INSENSITIVE",       RW_INSENSITIVE);
  tReservedWordMap.bind ("INSERT",            RW_INSERT);
  tReservedWordMap.bind ("INTEGER",           RW_INTEGER);
  tReservedWordMap.bind ("INTERSECT",         RW_INTERSECT);
  tReservedWordMap.bind ("INTERVAL",          RW_INTERVAL);
  tReservedWordMap.bind ("INTO",              RW_INTO);
  tReservedWordMap.bind ("INT",               RW_INT);
  tReservedWordMap.bind ("IN",                RW_IN);
  tReservedWordMap.bind ("ISOLATION",         RW_ISOLATION);
  tReservedWordMap.bind ("IS",                RW_IS);
  tReservedWordMap.bind ("JOIN",              RW_JOIN);
  tReservedWordMap.bind ("KEY",               RW_KEY);
  tReservedWordMap.bind ("LAST",              RW_LAST);
  tReservedWordMap.bind ("LEADING",           RW_LEADING);
  tReservedWordMap.bind ("LEFT",              RW_LEFT);
  tReservedWordMap.bind ("LEVEL",             RW_LEVEL);
  tReservedWordMap.bind ("LIKE",              RW_LIKE);
  tReservedWordMap.bind ("LOCAL",             RW_LOCAL);
  tReservedWordMap.bind ("LOWER",             RW_LOWER);
  tReservedWordMap.bind ("MATCH",             RW_MATCH);
  tReservedWordMap.bind ("MAX",               RW_MAX);
  tReservedWordMap.bind ("MIN",               RW_MIN);
  tReservedWordMap.bind ("MINUTE",            RW_MINUTE);
  tReservedWordMap.bind ("MODULE",            RW_MODULE);
  tReservedWordMap.bind ("MONTH",             RW_MONTH);
  tReservedWordMap.bind ("NAMES",             RW_NAMES);
  tReservedWordMap.bind ("NATIONAL",          RW_NATIONAL);
  tReservedWordMap.bind ("NATURAL",           RW_NATURAL);
  tReservedWordMap.bind ("NCHAR",             RW_NCHAR);
  tReservedWordMap.bind ("NEXT",              RW_NEXT);
  tReservedWordMap.bind ("NOT",               RW_NOT);
  tReservedWordMap.bind ("NO",                RW_NO);
  tReservedWordMap.bind ("NULLIF",            RW_NULLIF);
  tReservedWordMap.bind ("NULL",              RW_NULL);
  tReservedWordMap.bind ("NUMERIC",           RW_NUMERIC);
  tReservedWordMap.bind ("OCTET_LENGTH",      RW_OCTET_LENGTH);
  tReservedWordMap.bind ("OF",                RW_OF);
  tReservedWordMap.bind ("ONLY",              RW_ONLY);
  tReservedWordMap.bind ("ON",                RW_ON);
  tReservedWordMap.bind ("OPEN",              RW_OPEN);
  tReservedWordMap.bind ("OPTION",            RW_OPTION);
  tReservedWordMap.bind ("ORDER",             RW_ORDER);
  tReservedWordMap.bind ("OR",                RW_OR);
  tReservedWordMap.bind ("OUTER",             RW_OUTER);
  tReservedWordMap.bind ("OUTPUT",            RW_OUTPUT);
  tReservedWordMap.bind ("OVERLAPS",          RW_OVERLAPS);
  tReservedWordMap.bind ("PARTIAL",           RW_PARTIAL);
  tReservedWordMap.bind ("POSITION",          RW_POSITION);
  tReservedWordMap.bind ("PRECISION",         RW_PRECISION);
  tReservedWordMap.bind ("PREPARE",           RW_PREPARE);
  tReservedWordMap.bind ("PRESERVE",          RW_PRESERVE);
  tReservedWordMap.bind ("PRIMARY",           RW_PRIMARY);
  tReservedWordMap.bind ("PRIOR",             RW_PRIOR);
  tReservedWordMap.bind ("PRIVILEGES",        RW_PRIVILEGES);
  tReservedWordMap.bind ("PUBLIC",            RW_PUBLIC);
  tReservedWordMap.bind ("READ",              RW_READ);
  tReservedWordMap.bind ("REAL",              RW_REAL);
  tReservedWordMap.bind ("REFERENCES",        RW_REFERENCES);
  tReservedWordMap.bind ("RELATIVE",          RW_RELATIVE);
  tReservedWordMap.bind ("RESTRICT",          RW_RESTRICT);
  tReservedWordMap.bind ("REVOKE",            RW_REVOKE);
  tReservedWordMap.bind ("RIGHT",             RW_RIGHT);
  tReservedWordMap.bind ("ROLLBACK",          RW_ROLLBACK);
  tReservedWordMap.bind ("ROWS",              RW_ROWS);
  tReservedWordMap.bind ("SCHEMA",            RW_SCHEMA);
  tReservedWordMap.bind ("SCROLL",            RW_SCROLL);
  tReservedWordMap.bind ("SECOND",            RW_SECOND);
  tReservedWordMap.bind ("SELECT",            RW_SELECT);
  tReservedWordMap.bind ("SESSION_USER",      RW_SESSION_USER);
  tReservedWordMap.bind ("SESSION",           RW_SESSION);
  tReservedWordMap.bind ("SET",               RW_SET);
  tReservedWordMap.bind ("SIZE",              RW_SIZE);
  tReservedWordMap.bind ("SMALLINT",          RW_SMALLINT);
  tReservedWordMap.bind ("SOME",              RW_SOME);
  tReservedWordMap.bind ("SQLERROR",          RW_SQLERROR);
  tReservedWordMap.bind ("SQL",               RW_SQL);
  tReservedWordMap.bind ("SUBSTRING",         RW_SUBSTRING);
  tReservedWordMap.bind ("SUM",               RW_SUM);
  tReservedWordMap.bind ("SYSTEM_USER",       RW_SYSTEM_USER);
  tReservedWordMap.bind ("TABLE",             RW_TABLE);
  tReservedWordMap.bind ("TEMPORARY",         RW_TEMPORARY);
  tReservedWordMap.bind ("THEN",              RW_THEN);
  tReservedWordMap.bind ("TIMESTAMP",         RW_TIMESTAMP);
  tReservedWordMap.bind ("TIMEZONE_HOUR",     RW_TIMEZONE_HOUR);
  tReservedWordMap.bind ("TIMEZONE_MINUTE",   RW_TIMEZONE_MINUTE);
  tReservedWordMap.bind ("TIME",              RW_TIME);
  tReservedWordMap.bind ("TO",                RW_TO);
  tReservedWordMap.bind ("TRAILING",          RW_TRAILING);
  tReservedWordMap.bind ("TRANSACTION",       RW_TRANSACTION);
  tReservedWordMap.bind ("TRANSLATE",         RW_TRANSLATE);
  tReservedWordMap.bind ("TRANSLATION",       RW_TRANSLATION);
  tReservedWordMap.bind ("TRIM",              RW_TRIM);
  tReservedWordMap.bind ("TRUE",              RW_TRUE);
  tReservedWordMap.bind ("UNION",             RW_UNION);
  tReservedWordMap.bind ("UNIQUE",            RW_UNIQUE);
  tReservedWordMap.bind ("UNKNOWN",           RW_UNKNOWN);
  tReservedWordMap.bind ("UPDATE",            RW_UPDATE);
  tReservedWordMap.bind ("UPPER",             RW_UPPER);
  tReservedWordMap.bind ("USAGE",             RW_USAGE);
  tReservedWordMap.bind ("USER",              RW_USER);
  tReservedWordMap.bind ("USING",             RW_USING);
  tReservedWordMap.bind ("VALUES",            RW_VALUES);
  tReservedWordMap.bind ("VALUE",             RW_VALUE);
  tReservedWordMap.bind ("VARCHAR",           RW_VARCHAR);
  tReservedWordMap.bind ("VARYING",           RW_VARYING);
  tReservedWordMap.bind ("VIEW",              RW_VIEW);
  tReservedWordMap.bind ("WHENEVER",          RW_WHENEVER);
  tReservedWordMap.bind ("WHEN",              RW_WHEN);
  tReservedWordMap.bind ("WHERE",             RW_WHERE);
  tReservedWordMap.bind ("WITH",              RW_WITH);
  tReservedWordMap.bind ("WORK",              RW_WORK);
  tReservedWordMap.bind ("WRITE",             RW_WRITE);
  tReservedWordMap.bind ("YEAR",              RW_YEAR);

}  /* TApplication() */


TApplication::~TApplication (void)
{

  closeFiles();
  if ( !gCompilationOk && ( yyout != stdout ) )
  {
    remove (stCfgOutputFileName.data());
  }

}  /* ~TApplication() */


void TApplication::initialize (int iPARAMETERS_COUNT, const char** ppkcPARAMETERS_LIST)
{

  tConfigProcessor.setCommandLineParameters (iPARAMETERS_COUNT, ppkcPARAMETERS_LIST);

  /*
  *  Create options with their default values.
  */
  tConfigProcessor.addOption ("output",           "o", "out.cc");
  tConfigProcessor.addOption ("debug",            "d", "0");
  tConfigProcessor.addOption ("target",           "t", "");
  tConfigProcessor.addOption ("cdml-path",        "m", "/usr/share/uesqlc:/usr/local/share/uesqlc");
  tConfigProcessor.addOption ("help",             "h");
  tConfigProcessor.addOption ("version",          "V");
  tConfigProcessor.addOption ("parameters",       "p");
  tConfigProcessor.addOption ("output-cdml",      "u");
  tConfigProcessor.addOption ("output-cdml-skel", "s");
  tConfigProcessor.addOption ("line-directives",  "l");
  tConfigProcessor.addOption ("full-sql",         "f");
  tConfigProcessor.addOption ("intermediate-sql", "i");
  tConfigProcessor.addOption ("entry-sql",        "e");
  tConfigProcessor.processOptions();

  /*
  *  Get arguments, if any.
  */
  if ( tConfigProcessor.hasArguments() )
  {
    stCfgInputFileName = tConfigProcessor.argumentValue (0);
  }

  /*
  *  Get options values.
  */
  stCfgOutputFileName = tConfigProcessor ["output"].value();
  stCfgTarget         = tConfigProcessor ["target"].value();
  stCfgCdmlPath       = tConfigProcessor ["cdml-path"].value();
  uiCfgDebug          = tConfigProcessor ["debug"].value().toUnsignedInt();
  gCfgHelp            = ( tConfigProcessor ["help"].isAtCmdLine() );
  gCfgParameters      = ( tConfigProcessor ["parameters"].isAtCmdLine() );
  gCfgVersion         = ( tConfigProcessor ["version"].isAtCmdLine() );
  gCfgOutputCdml      = ( tConfigProcessor ["output-cdml"].isAtCmdLine() );
  gCfgOutputCdmlSkel  = ( tConfigProcessor ["output-cdml-skel"].isAtCmdLine() );
  gCfgLineDirectives  = ( tConfigProcessor ["line-directives"].isAtCmdLine() );
  if ( tConfigProcessor ["full-sql"].isAtCmdLine() )
  {
    tRequirementVerifier.clearConformanceLevelSet();
    tRequirementVerifier.addConformanceLevel (FULL_SQL_LEVEL);
  }
  else if ( tConfigProcessor ["intermediate-sql"].isAtCmdLine() )
  {
    tRequirementVerifier.clearConformanceLevelSet();
    tRequirementVerifier.addConformanceLevel (FULL_SQL_LEVEL);
    tRequirementVerifier.addConformanceLevel (INTERMEDIATE_SQL_LEVEL);
  }
  else if ( tConfigProcessor ["entry-sql"].isAtCmdLine() )
  {
    tRequirementVerifier.clearConformanceLevelSet();
    tRequirementVerifier.addConformanceLevel (FULL_SQL_LEVEL);
    tRequirementVerifier.addConformanceLevel (INTERMEDIATE_SQL_LEVEL);
    tRequirementVerifier.addConformanceLevel (ENTRY_SQL_LEVEL);
  }
  /*
  *  Or else defaults to Full SQL Level.
  */

}  /* initialize() */


int TApplication::start (void)
  throw (TExceptionUESQLC_Error)
{

  int    iExitCode = -1;
  bool   gContinue = true;
  
  try
  {
    if ( gContinue && gCfgVersion )
    {
      cout << release() << endl;
      gContinue = false;
    }
    if ( gContinue && gCfgParameters )
    {
      printParameters();
      gContinue = false;
    }
    if ( gContinue && gCfgHelp )
    {
      printUsage();
      gContinue = false;
    }
    if ( gContinue )
    {
      switch (uiCfgDebug)
      {
        case 0:
        case 1:
        {
          break;
        }
        case 2:
        {
          yydebug = 1;
          break;
        }
        default:
        {
          throw TExceptionUESQLC_Fatal ("bad value for debug level");
        }
      }
    }

    /*
    *  Checks number of arguments.
    */
    if ( tConfigProcessor.numberOfArguments() > 1 )
    {
      throw TExceptionUESQLC_Fatal ("wrong number of arguments");
    }
    if ( gContinue )
    {
      if ( gCfgOutputCdml )
      {
        if ( stCfgTarget.isEmpty() )
        {
          throw TExceptionUESQLC_Fatal ("there is no target");
        }
        else
        {
          setCodeGenerator (stCfgTarget.data());
          cout << *tTargetToCodeGeneratorMap [stCfgTarget] << endl;
          gContinue = false;
        }
      }
      else if ( gCfgOutputCdmlSkel )
      {
        if ( stCfgTarget.isEmpty() )
        {
          throw TExceptionUESQLC_Fatal ("there is no target");
        }
        else
        {
          tTargetToCodeGeneratorMap [stCfgTarget]->clearCode();
          cout << *tTargetToCodeGeneratorMap [stCfgTarget] << endl;
          gContinue = false;
        }
      }
      else
      {
        openFiles();
        iExitCode      = yyparse();
        gCompilationOk = ( iExitCode == 0 );

        /*
        *  If fwrite() returns 1, then it means that it could write one item
        *  of stCode.length() bytes.
        */

        /*
        *  Write prefix output code.
        */
        if ( !stPrefixOutputCode.isEmpty() )
        {
          if ( fwrite (stPrefixOutputCode.data(), stPrefixOutputCode.length(), 1, yyout) != 1 )
          {
            throw TExceptionUESQLC_Fatal ("couldn't write output file");
          }
        }

        /*
        *  Write body output code.
        */
        if ( fwrite (stOutputCode.data(), stOutputCode.length(), 1, yyout) != 1 )
        {
          throw TExceptionUESQLC_Fatal ("couldn't write output file");
        }

        /*
        *  Write postfix output code.
        */
        if ( !stPostfixOutputCode.isEmpty() )
        {
          if ( fwrite (stPostfixOutputCode.data(), stPostfixOutputCode.length(), 1, yyout) != 1 )
          {
            throw TExceptionUESQLC_Fatal ("couldn't write output file");
          }
        }

        /*
        *  Create complementary files (if any).
        */
        createComplementaryFiles();
      }
    }
  }
  catch (const TExceptionUESQLC_Error& rktSOURCE_EXCEPTION)
  {
    cerr << rktSOURCE_EXCEPTION;
  }  
  catch (const TExceptionUESQLC_Fatal& rktSOURCE_EXCEPTION)
  {
    cerr << rktSOURCE_EXCEPTION;
  }  
  catch (const TExceptionPBCG_NotCdmlFile& rktSOURCE_EXCEPTION)
  {
    cerr << rktSOURCE_EXCEPTION;
  }
  catch (const TExceptionPBCG_BadInstantiateSyntax& rktSOURCE_EXCEPTION)
  {
    cerr << rktSOURCE_EXCEPTION;
  }
  catch (const TExceptionPBCG_FileNotFound& rktSOURCE_EXCEPTION)
  {
    cerr << rktSOURCE_EXCEPTION;
  }
  catch (const TExceptionADT_Storage& rktSOURCE_EXCEPTION)
  {
    cerr << rktSOURCE_EXCEPTION;
  }
  catch (const TExceptionADT_FreezedInstance& rktSOURCE_EXCEPTION)
  {
    cerr << rktSOURCE_EXCEPTION;
  }
  catch (const TExceptionADT_NotIncluded& rktSOURCE_EXCEPTION)
  {
    cerr << rktSOURCE_EXCEPTION;
  }
  catch (const TExceptionADT_Constraint& rktSOURCE_EXCEPTION)
  {
    cerr << rktSOURCE_EXCEPTION;
  }
  catch (const TExceptionCP_FileError& rktSOURCE_EXCEPTION)
  {
    cerr << rktSOURCE_EXCEPTION;
  }
  catch (const TExceptionCP_VariableNotFound& rktSOURCE_EXCEPTION)
  {
    cerr << rktSOURCE_EXCEPTION;
  }
  catch (const TExceptionCP_NotParameterAble& rktSOURCE_EXCEPTION)
  {
    cerr << rktSOURCE_EXCEPTION.generalDescription();
    if ( rktSOURCE_EXCEPTION.specificDescription() )
    {
      cerr << " (--" << rktSOURCE_EXCEPTION.specificDescription() << ")";
    }
    cerr << endl;
  }
  catch (const TExceptionRV_Violation& rktSOURCE_EXCEPTION)
  {
    TString   stMessage;
    TString   stViolatedLevel;
    
    switch (rktSOURCE_EXCEPTION.iViolatedConformanceLevel)
    {
      case FULL_SQL_LEVEL:
      {
        /*
        *  Full SQL (High conformance).
        *
        *  The complete database language specified in this
        *  International Standard.
        */
        stViolatedLevel = "Full SQL level violated";
	break;
      }
      case INTERMEDIATE_SQL_LEVEL:
      {
        /*
        *  Intermediate SQL (Intermediate conformance).
        *
        *  Intermediate SQL is a subset of Full SQL as
        *  specified in the Leveling Rules.
        */
        stViolatedLevel = "Intermediate SQL level violated";
	break;
      }      
      case ENTRY_SQL_LEVEL:
      {
        /*
        *  Entry SQL (Low conformance).
        *
        *  Entry SQL is a subset of Intermediate SQL as spec-
        *  ified in the Leveling Rules.
        */
        stViolatedLevel = "Entry SQL level violated";
	break;
      }      
    }
    stMessage.format ( "%s: %d: near '%s'; %s: %s\n"             ,
                       stCfgInputFileName.data()                 ,
                       yylineno                                  ,
                       stLastToken.data()                        ,
                       stViolatedLevel.data()                    ,
                       rktSOURCE_EXCEPTION.specificDescription() );
    cerr << stMessage.data();
  }  
  return iExitCode;

}  /* start() */


void TApplication::outputCode (const char* pkcTEXT, size_t zLENGTH)
{

  if ( zLENGTH )
  {
    stOutputCode += TString (pkcTEXT, zLENGTH);
  }
  else
  {
    stOutputCode += pkcTEXT;
  }

}  /* outputCode() */


void TApplication::preAppendClauseCode (const char* pkcINSTRUCTION_NAME, const char* pkcCLAUSE_NAME)
{

  stPrefixOutputCode += getClauseCode (pkcINSTRUCTION_NAME, pkcCLAUSE_NAME);

}  /* preAppendClauseCode() */


void TApplication::postAppendClauseCode (const char* pkcINSTRUCTION_NAME, const char* pkcCLAUSE_NAME)
{

  stPostfixOutputCode += getClauseCode (pkcINSTRUCTION_NAME, pkcCLAUSE_NAME);

}  /* postAppendClauseCode() */


void TApplication::outputClauseCode (const char* pkcINSTRUCTION_NAME, const char* pkcCLAUSE_NAME)
{

  stOutputCode += getClauseCode (pkcINSTRUCTION_NAME, pkcCLAUSE_NAME);

}  /* outputClauseCode() */


void TApplication::setCodeGenerator (const char* pkcTARGET)
{

  TString   stNextTarget = pkcTARGET;

  gReusingTarget = ( tTargetToCodeGeneratorMap.isBinded (stNextTarget) );
  if ( !gReusingTarget )
  {
    tTargetToRuntimeParametersMapMap.bind (stNextTarget, TVariableMap());
    tTargetToCodeGeneratorMap.bind ( stNextTarget                                                        ,
                                     new TPatternBasedCodeGenerator ( (stNextTarget + ".cdmlo").data() ,
                                                                      stCfgCdmlPath.data()             ) );
  }
  stCurrentTarget        = stNextTarget;
  ptRuntimeParametersMap = &tTargetToRuntimeParametersMapMap [stCurrentTarget];

}  /* setCodeGenerator() */


void TApplication::initCodeParameters (void)
{

  /*
  *  Initiate code parameters from CDML variables.
  */
  tCodeParametersMap.clear();
  tCodeParametersMap.bind ( "__target__"                                                  ,
                            TVariableString (stCurrentTarget, TVariableString::eOptional) );
  if ( !ptRuntimeParametersMap->isEmpty() )
  {
    register size_t   zCardinal = ptRuntimeParametersMap->linksCardinal();
    
    for (register size_t J = 0; ( J < zCardinal ) ;J++)
    {
      tCodeParametersMap.bind (ptRuntimeParametersMap->key (J), ptRuntimeParametersMap->item (J));
    }
  }

}  /* initCodeParameters() */


void TApplication::pushTokenName (const char* pkcLAST_TOKEN)
{

  stLastToken = pkcLAST_TOKEN;  

}  /* pushTokenName() */


int TApplication::identifyToken (const char* pkcTEXT)
{

  int       iToken;
  TString   stTokenName;
  TString   stToken = pkcTEXT;

  stToken.toUppercase();
  if ( tReservedWordMap.isBinded (stToken) )
  {
    iToken = tReservedWordMap [stToken];
    pushTokenName ((TString ("RW_") + stToken).data());
  }
  else
  {
    iToken                = IDENTIFIER_BODY;
    yylval.stCode         = stToken;
    yylval.stOriginalText = pkcTEXT;
    pushTokenName ("IDENTIFIER_BODY");
  }
  printDebugInformation();
  return iToken;

}  /* identifyToken() */


void TApplication::handleTarget (const char* pkcTEXT)
{

  /*
  *  Saves the line number for the embedded introducer.
  */
  zEmbeddedIntroducerLine = yylineno;

  if ( !stCfgTarget.isEmpty() )
  {
    /*
    *  Loads CDML file.
    */
    setCodeGenerator (stCfgTarget.data());
  }
  else
  {
    /*
    *  If there is no global target, then load the proper target.
    */

    size_t    zBegin;
    size_t    zEnd;
    TString   stText (pkcTEXT);

    /*
    *  Remove one-line comments from text.
    */
    while ( stText.findFirst ("//", zBegin) )
    {
      stText.findFirst ("\n", zEnd, zBegin);
      stText.removeSubstring (zBegin, (zEnd - zBegin) + 1);
    }
    
    /*
    *  Extract target.
    */
    stText.findFirst ('"', zBegin);
    zBegin += strlen ("UESQL::") + 1;
    stText.findFirst ('"', zEnd, zBegin);
    stText = stText (zBegin, zEnd - zBegin);
    if ( stText == "default" )
    {
      throw TExceptionUESQLC_Fatal ("there is no target");
    }
    else
    {
      /*
      *  Loads CDML file.
      */
      setCodeGenerator (stText.data());
    }
  }
  if ( ptRuntimeParametersMap->isEmpty() )
  {
    *ptRuntimeParametersMap = tTargetToCodeGeneratorMap [stCurrentTarget]->variables();
  }
  
}  /* handleTarget() */


TString TApplication::collapseStringLiteral (const char* pkcTEXT)
{

  TString       stCollapsedStringLiteral;
  bool          gBetweenParts;
  char          cQuoteCharacter;
  const char*   pkcIterator = pkcTEXT;

  /*
  *  If the first character is 'N', 'B' or 'X', then
  *  let's eat that character.
  */
  if ( ( *pkcIterator == 'N' ) || ( *pkcIterator == 'B' ) || ( *pkcIterator == 'X' ) )
  {
    ++pkcIterator;
  }

  /*
  *  Select quote/double-quote character.
  */
  cQuoteCharacter = *pkcIterator;

  /*
  *  Eat the quote/double-quote character.
  */
  ++pkcIterator;

  if ( cQuoteCharacter == '"' )
  {
    /*
    *  <delimited identifier> ::= <double quote> <delimited identifier body> <double quote>
    */
    stCollapsedStringLiteral = pkcTEXT + 1;
    stCollapsedStringLiteral.replaceAll ("\"\"", "\"");
    stCollapsedStringLiteral [stCollapsedStringLiteral.length() - 1] = '\0';
  }
  else
  {
    /*
    *  <character string literal> ::= [ <introducer><character set specification> ]
    *                                 <quote> [ <character representation>... ] <quote>
    *                                 [ { <separator>... <quote> [ <character representation>... ] <quote> }... ]
    *
    *  <national character string literal> ::= N <quote> [ <character representation>... ] <quote>
    *                                          [ { <separator>... <quote> [ <character representation>... ] <quote> }... ]
    *
    *  <bit string literal> ::= B <quote> [ <bit>... ] <quote>
    *                           [ { <separator>... <quote> [ <bit>... ] <quote> }... ]
    *
    *  <hex string literal> ::= X <quote> [ <hexit>... ] <quote>
    *                           [ { <separator>... <quote> [ <hexit>... ] <quote> }... ]
    */
 
    /*
    *  Copy (collapsing if needed) the rest of the string.
    */
    while ( *pkcIterator )
    {
      if ( *pkcIterator != cQuoteCharacter )
      {
        stCollapsedStringLiteral << *pkcIterator;
        ++pkcIterator;
      }
      else
      {
        /*
        *  Eat quote/double-quote character.
        */
        ++pkcIterator;

        /*
        *  If there is anything after quote, then it is a multipart string_literal.
        *  The only thing that can be between string_literal parts, is {separator}+.
        *  Where {separator} ::= ({comment}|{space}|{newline}|{tabulator})+
        */
        if ( *pkcIterator )
        {
          gBetweenParts = true;
          while ( gBetweenParts )
          {
            switch (*pkcIterator)
            {
              case '-':
              {
                /*
                *  Eat {comment} terminal.
                */
                ++pkcIterator;
                while ( *pkcIterator != '\n' ) ++pkcIterator;
                break;
              }
              case ' ':
              case '\t':
              case '\n':
              {
                /*
                *  Eat {tabulator}, {space} and {newline}.
                */
                ++pkcIterator;
                break;
              }
              case '\'':
              {
                /*
                *  Exit loop whenever a new string_literal part begins.
                */
                gBetweenParts = false;
                ++pkcIterator;
                break;
              }
              default:
              {
                throw TExceptionUESQLC_Error ("bad charater between string_literal parts", __FILE__, __LINE__);
              }
            }
          }
        }
      }
    }
  }
  return stCollapsedStringLiteral;
  
}  /* collapseStringLiteral() */


/*
*  S E L E C T O R S
*/

bool TApplication::isReusingTarget (void) const
{

  return gReusingTarget;

}  /* isReusingTarget() */


bool TApplication::useLineDirective (void) const
{

  return gCfgLineDirectives;

}  /* useLineDirective() */


const TString TApplication::programFileName (void) const
{

  return tConfigProcessor.programFileName();

}  /* programFileName() */


TString TApplication::getLineDirective (bool gEMBEDDED_INTRODUCER_LINE_NUMBER) const
{

  TString   stLineDirective;
  
  if ( gEMBEDDED_INTRODUCER_LINE_NUMBER )
  {
    stLineDirective.format ( "#line %u \"%s\"\n"                    ,
                             (unsigned int) zEmbeddedIntroducerLine ,
                             stCfgInputFileName.data()              );
  }
  else
  {
    stLineDirective.format ( "#line %d \"%s\"\n"       ,
                             yylineno                  ,
                             stCfgInputFileName.data() );
  }
  return stLineDirective;

}  /* getLineDirective() */


TString TApplication::getClauseCode (const char* pkcINSTRUCTION_NAME, const char* pkcCLAUSE_NAME) const
{

  TString*                                            pstTargetParameterValue;
  TString                                             stTargetParameter;
  TString                                             stCode;
  bool                                                gParameterCheck;
  register size_t                                     zClauseArraySize;
  TLinkedMap<TString, const TBaseClauseBodyString*>   tParameterToCodeMap;
  TBaseInstructionBody&                               rtInstructionBody ((*tTargetToCodeGeneratorMap [stCurrentTarget]) [pkcINSTRUCTION_NAME]);

  if ( uiCfgDebug > 0 )
  {
    cerr << programFileName() << ": writing code for <instruction, clause> pair: "
         << pkcINSTRUCTION_NAME << ", " << pkcCLAUSE_NAME << endl;
    if ( uiCfgDebug > 1 )
    {
      cerr << programFileName() << ": code:\n" << rtInstructionBody.clauseBody (pkcCLAUSE_NAME);
    }
  }

  gParameterCheck = rtInstructionBody.clauseTag (pkcCLAUSE_NAME).mustCheckParameters();
  stCode          = rtInstructionBody.clauseBody (pkcCLAUSE_NAME).instantiate (tCodeParametersMap, gParameterCheck);

  /*
  *  Update targets for the other targeted clauses.
  */
  tParameterToCodeMap = rtInstructionBody.targetedVariablesForClause (pkcCLAUSE_NAME);
  zClauseArraySize    = tParameterToCodeMap.linksCardinal();

  for (register size_t J = 0; ( J < zClauseArraySize ) ;J++)
  {
    /*
    *  First obtain the parameter name.
    */
    stTargetParameter = tParameterToCodeMap.key (J);

    /*
    *  Then build an auxiliary clause body with the
    *  parameter name. Next instantiate it (substitute)
    *  with runtime-code-parameters (if any).
    */
    stTargetParameter = TClauseBodyString (stTargetParameter.data()).instantiate (*ptRuntimeParametersMap, false);

    /*
    *  Updates the value of target parameter value on
    *  runtime-code-parameters-map and on code-parameters-map,
    *  with the instantiation of the former clause.
    */
    pstTargetParameterValue  = &((*ptRuntimeParametersMap) [stTargetParameter]);
    *pstTargetParameterValue = tParameterToCodeMap.item (J)->instantiate (tCodeParametersMap, false);
    tCodeParametersMap [stTargetParameter] = *pstTargetParameterValue;
  }
  return stCode;

}  /* getClauseCode() */


void TApplication::require (long int liREQUERIMENT_IDENTIFIER...) const
{

  va_list   tVa_list;

  va_start (tVa_list, liREQUERIMENT_IDENTIFIER);
  tRequirementVerifier.require (liREQUERIMENT_IDENTIFIER, tVa_list);
  va_end (tVa_list);

}  /* require() */


void TApplication::handleCompilerError (const char* pkcMESSAGE) const
  throw (TExceptionUESQLC_Error)
{

  TString   stMessage;

  stMessage.format ("near '%s'; %s", stLastToken.data(), pkcMESSAGE);
  throw TExceptionUESQLC_Error (stMessage.data(), stCfgInputFileName.data(), yylineno);

}  /* handleCompilerError() */


void TApplication::createComplementaryFiles (void) const
  throw (TExceptionUESQLC_Error)
{

  TVariableMap*     ptVariableMap;
  register size_t   zVariablesMapSize;
  ofstream          tOutputOfstream;
  TString           stKey;
  register size_t   zTargetsMapSize = tTargetToRuntimeParametersMapMap.linksCardinal();
//  const char*       pkcPrefix       = "__file::";
//  const size_t      kzPrefixSize    = strlen (pkcPrefix);

  for (register size_t I = 0; ( I < zTargetsMapSize ) ;I++)
  {
    ptVariableMap     = &tTargetToRuntimeParametersMapMap.item (I);
    zVariablesMapSize = ptVariableMap->linksCardinal();
    for (register size_t J = 0; ( J < zVariablesMapSize ) ;J++)
    {
      if ( ptVariableMap->item (J).type() == TVariableString::eFile )
      {
        tOutputOfstream.open (ptVariableMap->key (J).data());
        if ( tOutputOfstream.good() )
        {
          tOutputOfstream << ptVariableMap->item (J);
          tOutputOfstream.close();
        }
        else
        {
          throw TExceptionUESQLC_Fatal ("couldn't write complementary file");
        }
      }
//       stKey = ptVariableMap->key (J);
//       if ( stKey (0, kzPrefixSize) == pkcPrefix )
//       {
//         stKey.removeSubstring (0, kzPrefixSize);
//         tOutputOfstream.open (stKey.data());
//         if ( tOutputOfstream.good() )
//         {
//           tOutputOfstream << ptVariableMap->item (J);
//           tOutputOfstream.close();
//         }
//         else
//         {
//           throw TExceptionUESQLC_Fatal ("couldn't write complementary file");
//         }
//       }
    }
  }
  
}  /* createComplementaryFiles() */


TString TApplication::debugDescription (void) const
{

  return TString().format ( "%s(%d): parsed terminal '%s' as [%s]\n" ,
                            stCfgInputFileName.data()                ,
                            yylineno                                 ,
                            stLastToken.data()                       ,
                            yytext                                   );

}  /* debugDescription() */


void TApplication::printDebugInformation (void) const
{

  if ( uiCfgDebug > 0 )
  {
    cerr << debugDescription();
  }

}  /* printDebugInformation() */


void TApplication::printUsage (void) const
{

  TString   stUsageInfo;
  
  stUsageInfo.format ( "%s Universal Embedded SQL Compiler Version %s\n"
                       "Copyright (c) Rafael Jess Alcntara Prez, 1996-1998.\n"
                       "Usage:  %s [OPTIONS]... [FILE]\n\n"
                       "  -%s, --%s=TARGET_PLATFORM\n"
                       "  -%s, --%s=OUTPUT_FILE_NAME\n"
                       "  -%s, --%s=CDML_PATH\n"
                       "  -%s, --%s={0,1,2}\n"
                       "  -%s, --%s\n\n"
                       "Conformance Level selection:\n"
                       "  -%s, --%s (default)\n"
                       "  -%s, --%s\n"
                       "  -%s, --%s\n\n"
                       "Informative output:\n"
                       "  -%s, --%s\n"
                       "  -%s, --%s\n"
                       "  -%s, --%s\n"
                       "  -%s, --%s (need target)\n"
                       "  -%s, --%s (need target)\n\n"                         ,
                       name().data()                                           ,
                       release().data()                                        ,
                       programFileName().data()                                ,
                       tConfigProcessor ["target"].shortcut().data()           ,
                       tConfigProcessor ["target"].name().data()               ,
                       tConfigProcessor ["output"].shortcut().data()           ,
                       tConfigProcessor ["output"].name().data()               ,
                       tConfigProcessor ["cdml-path"].shortcut().data()        ,
                       tConfigProcessor ["cdml-path"].name().data()            ,
                       tConfigProcessor ["debug"].shortcut().data()            ,
                       tConfigProcessor ["debug"].name().data()                ,
                       tConfigProcessor ["line-directives"].shortcut().data()  ,
                       tConfigProcessor ["line-directives"].name().data()      ,
                       tConfigProcessor ["full-sql"].shortcut().data()         ,
                       tConfigProcessor ["full-sql"].name().data()             ,
                       tConfigProcessor ["intermediate-sql"].shortcut().data() ,
                       tConfigProcessor ["intermediate-sql"].name().data()     ,
                       tConfigProcessor ["entry-sql"].shortcut().data()        ,
                       tConfigProcessor ["entry-sql"].name().data()            ,
                       tConfigProcessor ["help"].shortcut().data()             ,
                       tConfigProcessor ["help"].name().data()                 ,
                       tConfigProcessor ["version"].shortcut().data()          ,
                       tConfigProcessor ["version"].name().data()              ,
                       tConfigProcessor ["parameters"].shortcut().data()       ,
                       tConfigProcessor ["parameters"].name().data()           ,
                       tConfigProcessor ["output-cdml"].shortcut().data()      ,
                       tConfigProcessor ["output-cdml"].name().data()          ,
                       tConfigProcessor ["output-cdml-skel"].shortcut().data() ,
                       tConfigProcessor ["output-cdml-skel"].name().data()     );
  cout << stUsageInfo;
  
}  /* printUsage() */


void TApplication::printParameters (void) const
{

  cout << "output:           " << stCfgOutputFileName << endl;
  cout << "input-file:       " << stCfgInputFileName  << endl;
  cout << "cdml-path:        " << stCfgCdmlPath       << endl;
  cout << "target:           " << stCfgTarget         << endl;
  cout << "debug:            " << uiCfgDebug          << endl;
  cout << "line-directives:  " << gCfgLineDirectives  << endl;
  cout << "help:             " << gCfgHelp            << endl;
  cout << "version:          " << gCfgVersion         << endl;
  cout << "parameters:       " << gCfgParameters      << endl;
  cout << "output-cdml:      " << gCfgOutputCdml      << endl;
  cout << "output-cdml-skel: " << gCfgOutputCdmlSkel  << endl;

}  /* printParameters() */


void TApplication::openFiles (void)
{

  /*
  *  Open input file name.
  */
  if ( stCfgInputFileName.isEmpty() )
  {
    throw TExceptionUESQLC_Fatal ("no input file");
  }
  else
  {
    yyin = fopen (stCfgInputFileName.data(), "r");
    if ( yyin == NULL )
    {
      TString   stMessage;

      stMessage.format ("couldn't open file '%s'", stCfgInputFileName.data());
      throw TExceptionUESQLC_Fatal (stMessage.data());
    }
  }

  /*
  *  Opens output file name.
  */
  if ( stCfgOutputFileName.isEmpty() || ( stCfgOutputFileName == "-" ) )
  {
    stCfgOutputFileName = "stdout";
    yyout               = stdout;
  }
  else
  {
    yyout = fopen (stCfgOutputFileName.data(), "w");
    if ( yyout == NULL )
    {
      throw TExceptionUESQLC_Fatal ("can't write output file");
    }
  }

}  /* openFiles() */


void TApplication::closeFiles (void)
{

  /*
  *  yyin is always distinct from stdin.
  */
  if ( yyin )
  {
    fclose (yyin);
  }
  if ( yyout && ( yyout != stdout ) )
  {
    fclose (yyout);
  }

}  /* closeFiles() */

