/*
*  Name:      requeriment_verifier.cc
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   UESQLC requirement verifier for ANSI/ISO SQL 92
*  Date:      $Date: 1998/11/23 11:21:39 $
*  Revision:  $Revision: 1.4 $
*
*  Copyright (C) 1996-1998  Rafael Jesus Alcantara Perez <ralcan@vnet.es>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#include <cctype>
#include "requirement_verifier.hh"


/*
*  G L O B A L   T E M P L A T E   E X P L I C I T   I N S T A N T I A T I O N S
*/

template class TCollection<TBaseRequirementVerifier<TStackValue*>::TRequirement>;
template class TBaseContainer<TBaseRequirementVerifier<TStackValue*>::TRequirement>;
template class TLinkedContainer<TBaseRequirementVerifier<TStackValue*>::TRequirement>;
template class TBaseMap<long int, TBaseRequirementVerifier<TStackValue*>::TRequirement>;
template class TLinkedMap<long int, TBaseRequirementVerifier<TStackValue*>::TRequirement>;
template class TBaseDynamicArray<TStackValue*>;
template class TLinkedDynamicArray<TStackValue*>;
template class TVariableArgumentArray<TStackValue*>;
template class TBaseRequirementVerifier<TStackValue*>;
template class TCollection<TStackValue*>;
template class TBaseContainer<TStackValue*>;
template class TLinkedContainer<TStackValue*>;
template class TBaseSet<TStackValue*>;
template class TLinkedSet<TStackValue*>;


/*
*  C O N S T R U C T O R S
*/

TRequirementVerifier::TRequirementVerifier (void)         :
  TBaseRequirementVerifier<TStackValue*> (FULL_SQL_LEVEL)
{

  addRequirement ( SQL92_SR_5_2_P8                                                ,
                   FULL_SQL_LEVEL                                                 ,
                   "<regular identifier> shall not has more than 128 characters"  );
  addRequirement ( SQL92_SR_5_2_P9                                                ,
                   FULL_SQL_LEVEL                                                 ,
                   "<delimited identifier body> shall not has more than 128 "
                   "characters"                                                   );
  addRequirement ( SQL92_SR_5_3_P21                                               ,
                   FULL_SQL_LEVEL                                                 ,
                   "within <datetime literal>, <years value> shall contain four "
                   "digits"                                                       );
  addRequirement ( SQL92_SR_8_2_P1                                                ,
                   FULL_SQL_LEVEL                                                 ,
                   "the <row value constructor>s shall be of the same degree"     );
  addRequirement ( SQL92_SR_11_3_P5                                               ,
                   FULL_SQL_LEVEL                                                 ,
                   "a <temporary table declaration> shall contain at least one "
                   "<column definition>"                                          );
  addRequirement ( SQL92_SR_13_11_P4                                              ,
                   FULL_SQL_LEVEL                                                 ,
                   "a <table definition> shall contain at least one "
                   "<column definition>"                                          );
  addRequirement ( SQL92_LR_5_2_P1                                                ,
                   INTERMEDIATE_SQL_LEVEL                                         ,
                   "no <identifier body> shall end in an <underscore>"            );
  addRequirement ( SQL92_LR_5_2_P2_A                                              ,
                   ENTRY_SQL_LEVEL                                                ,
                   "no <regular identifier> or <delimited identifier body> shall"
                   " contain more than 18 <character representation>s"            );
  addRequirement ( SQL92_LR_5_2_P2_B                                              ,
                   ENTRY_SQL_LEVEL                                                ,
                   "an <identifier body> shall contain no <simple Latin lower ca"
                   "se letter>"                                                   );
  addRequirement ( SQL92_LR_6_3_P2_B                                              ,
                   ENTRY_SQL_LEVEL                                                ,
                   "the optional <key word> AS shall not be specified"            );
  addRequirement ( SQL92_LR_6_3_P2_C                                              ,
                   ENTRY_SQL_LEVEL                                                ,
                   "<derived column list> shall not be specified"                 );
  addRequirement ( SQL92_LR_6_8_P1                                                ,
                   INTERMEDIATE_SQL_LEVEL                                         ,
                   "language shall contain no <time precision> or "
                   "<timestamp precision>"                                        );
  addRequirement ( SQL92_LR_6_8_P2                                                ,
                   ENTRY_SQL_LEVEL                                                ,
                   "language shall not contain any <datetime value function>"     );
  addRequirement ( SQL92_LR_6_11_P2_A                                             ,
                   ENTRY_SQL_LEVEL                                                ,
                   "a <value expression> shall not be a "
                   "<datetime value expression>"                                  );
  addRequirement ( SQL92_LR_6_11_P2_B                                             ,
                   ENTRY_SQL_LEVEL                                                ,
                   "a <value expression> shall not be a "
                   "<interval value expression>"                                  );
  addRequirement ( SQL92_LR_6_11_P2_C                                             ,
                   ENTRY_SQL_LEVEL                                                ,
                   "a <value expression primary> shall not be a "
                   "<case expression>"                                            );
  addRequirement ( SQL92_LR_6_11_P2_D                                             ,
                   ENTRY_SQL_LEVEL                                                ,
                   "a <value expression primary> shall not be a "
                   "<cast specification>"                                         );
  addRequirement ( SQL92_LR_8_9_P2                                                ,
                   ENTRY_SQL_LEVEL                                                ,
                   "language shall not contain any <unique predicate>"            );
  addRequirement ( SQL92_LR_8_10_P1                                               ,
                   INTERMEDIATE_SQL_LEVEL                                         ,
                   "language shall not contain any <match predicate>"             );
  addRequirement ( SQL92_LR_11_10_P2                                               ,
                   ENTRY_SQL_LEVEL                                                 ,
                   "language shall not contain any <alter table statement>"        );
  addRequirement ( SQL92_LR_13_1_P1_A                                              ,
                   INTERMEDIATE_SQL_LEVEL                                          ,
                   "a <declare cursor> shall not specify INSENSITIVE"              );
  addRequirement ( SQL92_LR_13_1_P2_A                                              ,
                   ENTRY_SQL_LEVEL                                                 ,
                   "a <declare cursor> shall not specify SCROLL"                   );
  addRequirement ( SQL92_LR_13_1_P2_B                                              ,
                   ENTRY_SQL_LEVEL                                                 ,
                   "a <cursor specification> shall not contain an "
                   "<updatability clause>"                                         );
  addRequirement ( SQL92_LR_13_3_P2_A                                              ,
                   ENTRY_SQL_LEVEL                                                 ,
                   "a <fetch statement> shall not specify a "
                   "<fetch orientation>"                                           );
  addRequirement ( SQL92_LR_13_3_P2_B                                              ,
                   ENTRY_SQL_LEVEL                                                 ,
                   "a <fetch statement> shall not specify FROM"                    );

}  /* TRequirementVerifier() */


/*
*  S E L E C T O R S
*/

bool TRequirementVerifier::verify (long int liREQUERIMENT_IDENTIFIER) const
{

  bool   gSuccess = false;

  switch (liREQUERIMENT_IDENTIFIER)
  {
    case SQL92_SR_5_2_P8:
    case SQL92_SR_5_2_P9:
    {
      gSuccess = ( tArgumentArray [0]->stCode.length() <= 128 );
      break;
    }
    case SQL92_SR_5_3_P21:
    {
      size_t   zOffset = 0;

      tArgumentArray [0]->stCode.findFirst ('-', zOffset);
      gSuccess = ( zOffset == 3 );
      break;
    }
    case SQL92_SR_8_2_P1:
    {
      gSuccess = ( tArgumentArray [0]->iItemCount == tArgumentArray [1]->iItemCount );
      break;
    }
    case SQL92_SR_11_3_P5:
    case SQL92_SR_13_11_P4:
    {
      gSuccess = ( tArgumentArray [0]->gHasColumnDefinition );
      break;
    }
    case SQL92_LR_5_2_P1:
    {
      TString&   rstCode        = tArgumentArray [0]->stCode;
      size_t     zLastCharacter = rstCode.length() - 1;
      
      gSuccess = ( rstCode [zLastCharacter] != '_' );
      break;
    }
    case SQL92_LR_5_2_P2_A:
    {
      gSuccess = ( tArgumentArray [0]->stCode.length() <= 18 );
      break;
    }
    case SQL92_LR_5_2_P2_B:
    {
      TString&   rstCode       = tArgumentArray [0]->stCode;
      size_t     zStringLength = rstCode.length();
      
      gSuccess = true;
      for (register size_t J = 0; ( J < zStringLength ) ;J++)
      {
	if ( islower (rstCode [J]) )
	{
	  gSuccess = false;
	  break;
	}
      }
      break;
    }
    case SQL92_LR_6_3_P2_B:
    case SQL92_LR_6_3_P2_C:
    case SQL92_LR_6_8_P1:
    case SQL92_LR_6_8_P2:
    case SQL92_LR_6_11_P2_A:
    case SQL92_LR_6_11_P2_B:
    case SQL92_LR_6_11_P2_C:
    case SQL92_LR_6_11_P2_D:
    case SQL92_LR_8_9_P2:
    case SQL92_LR_8_10_P1:
    case SQL92_LR_11_10_P2:
    case SQL92_LR_13_1_P1_A:
    case SQL92_LR_13_1_P2_A:
    case SQL92_LR_13_1_P2_B:
    case SQL92_LR_13_3_P2_A:
    case SQL92_LR_13_3_P2_B:
    {
      break;
    }
  }
  return gSuccess;

}  /* verify() */


