#include "oracle-oci-7.x.hh"


/*
*  UESQLC::oracle-oci-7.x internal variables initiation.
*/

TOci7xManager   htOci7xManager;


/*
*  Name:      test-base-1.uesql.cc
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Test for UESQL
*  Date:      $Date: 1998/11/23 11:24:06 $
*  Revision:  $Revision: 1.13 $
*
*  Copyright (C) 1996-1998  Rafael Jesus Alcantara Perez <ralcan@vnet.es>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#include <cstdlib>
#include <iostream>
#include <mpcl/test.h>
#include <mpcl/string.hh>
#include <mpcl/config_processor.hh>


TString GetErrorMessage (void)
{

  TString   stMessage;

  if ( SQLSTATE == "00000" )
  {
    stMessage = "successful completion";
  }
  else if ( SQLSTATE == "02000" )
  {
    stMessage = "no data";
  }
  else if ( SQLSTATE == "08001" )
  {
    stMessage = "connection exception: SQL-client unable to establish SQL-connection";
  }
  else if ( SQLSTATE == "08002" )
  {
    stMessage = "connection exception: connection name in use";
  }
  else if ( SQLSTATE == "08003" )
  {
    stMessage = "connection exception: connection does not exist";
  }
  else if ( SQLSTATE == "08007" )
  {
    stMessage = "connection exception: transaction resolution unknown";
  }
  else if ( SQLSTATE == "21000" )
  {
    stMessage = "cardinality violation";
  }
  else if ( SQLSTATE == "34000" )
  {
    stMessage = "invalid cursor name";
  }
  else if ( SQLSTATE == "40003" )
  {
    stMessage = "transaction rollback: statement completion unknown";
  }
  else if ( SQLSTATE == "42000" )
  {
    stMessage = "syntax error or access rule violation";
  }
  else
  {
    stMessage = "unknown error";
  }
  return stMessage;

}  /* GetErrorMessage() */


/*
*  Parameters and arguments:
*
*    {--hostname, -o}=HOSTNAME
*    {--username, -u}=USERNAME
*    {--password, -p}=PASSWORD
*/
int main (int argc, const char* argv[])
{

  TConfigProcessor   tConfig;
  size_t             zRecordCount = 0;

  TEST_INIT ("test-base-1");
  TEST_START_WATCH;

  TString   stVar0;
  TString   stVar1;
  TString   stVar2;
  TString   stAttribName;
  TString   stCursorName;
  TString   stUserName;
  TString   stHostName;
  TString   stCodeValue = "'002'";

  /*
  *  Define string constants names for options.
  */ 
  tConfig.addOption ("hostname", "o", "localhost");
  tConfig.addOption ("username", "u", getenv ("USER"));
  tConfig.addOption ("password", "p", "");
  tConfig.setCommandLineParameters (argc, argv);
  tConfig.processOptions();
  
  stHostName.format ("'%s'", tConfig ["hostname"].value().data());
  if ( tConfig ["password"].value().isEmpty() )
  {
    stUserName.format ("'%s'", tConfig ["username"].value().data());
  }
  else
  {
    stUserName.format ( "'%s/%s'"                           ,
                        tConfig ["username"].value().data() ,
                        tConfig ["password"].value().data() );
  }

  TEST_WRITE_INFO ("HOSTNAME = %s", stHostName.data());
  TEST_WRITE_INFO ("USER     = %s", stUserName.data());
  if ( !stHostName.isEmpty() && !stUserName.isEmpty() )
  {
{
/*
*  OCI 7.x instructions.
*/
if ( false )
{
  htOci7xManager.stSqlInstruction.format ("");
  OCI7X_EXEC (htOci7xManager.stSqlInstruction);
  OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto error;);
}
if ( false )
{
  htOci7xManager.stSqlInstruction.format ("");
  OCI7X_EXEC (htOci7xManager.stSqlInstruction);
  OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto error;);
}
{
TString   hstSqlServerName;
TString   hstConnectionName;
TString   hstUserName;
TString   hstUserId;

htOci7xManager.initiateEmbeddedVariableMap (NULL , stHostName.data(), stUserName.data(), NULL);
hstSqlServerName  = htOci7xManager.instantiateString ("%s");
hstConnectionName = htOci7xManager.kstDefaultConnectionName;
hstUserName       = htOci7xManager.instantiateString ("%s");
hstUserId.format ("%s@%s", hstUserName.data(), hstSqlServerName.data());
hstConnectionName.suppress ('\'');
hstUserId.suppress ('\'');
htOci7xManager.iCode = htOci7xManager.connect (hstConnectionName, (text*) hstUserId.data());
OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto error;);
}
if ( false )
{
  htOci7xManager.stSqlInstruction.format ("", stHostName.data(), stUserName.data());
  OCI7X_EXEC (htOci7xManager.stSqlInstruction);
  OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto error;);
}
if ( true )
{
  htOci7xManager.stSqlInstruction.format ("CREATE TABLE JOBS (CODE CHARACTER (3) NOT NULL, WORKER_COUNT INTEGER)");
  OCI7X_EXEC (htOci7xManager.stSqlInstruction);
  OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto error;);
}
if ( true )
{
  htOci7xManager.stSqlInstruction.format ("CREATE TABLE WORKERS (WORK_CODE CHARACTER (3) NOT NULL, NAME CHARACTER (40) NOT NULL, HOURS DECIMAL (5))");
  OCI7X_EXEC (htOci7xManager.stSqlInstruction);
  OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto error;);
}
if ( true )
{
  htOci7xManager.stSqlInstruction.format ("ALTER TABLE WORKERS ADD STATE CHARACTER (40)");
  OCI7X_EXEC (htOci7xManager.stSqlInstruction);
  OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto error;);
}
if ( true )
{
  htOci7xManager.stSqlInstruction.format ("ALTER TABLE WORKERS ADD PHONE_NUMBER INTEGER");
  OCI7X_EXEC (htOci7xManager.stSqlInstruction);
  OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto error;);
}
if ( true )
{
  htOci7xManager.stSqlInstruction.format ("DELETE FROM JOBS WHERE ( CODE = %s )", stCodeValue.data());
  OCI7X_EXEC (htOci7xManager.stSqlInstruction);
  OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto error;);
}
if ( true )
{
  htOci7xManager.stSqlInstruction.format ("DELETE FROM JOBS WHERE ( CODE = '001' )");
  OCI7X_EXEC (htOci7xManager.stSqlInstruction);
  OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto error;);
}
if ( true )
{
  htOci7xManager.stSqlInstruction.format ("DELETE FROM WORKERS");
  OCI7X_EXEC (htOci7xManager.stSqlInstruction);
  OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto error;);
}
if ( true )
{
  htOci7xManager.stSqlInstruction.format ("INSERT INTO JOBS VALUES ('001', 100)");
  OCI7X_EXEC (htOci7xManager.stSqlInstruction);
  OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto error;);
}
if ( true )
{
  htOci7xManager.stSqlInstruction.format ("INSERT INTO JOBS VALUES ('002', 250)");
  OCI7X_EXEC (htOci7xManager.stSqlInstruction);
  OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto error;);
}
if ( true )
{
  htOci7xManager.stSqlInstruction.format ("INSERT INTO WORKERS VALUES ('001', 'myname', 40, 'Andalucia', 5550000)");
  OCI7X_EXEC (htOci7xManager.stSqlInstruction);
  OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto error;);
}
if ( true )
{
  htOci7xManager.stSqlInstruction.format ("INSERT INTO WORKERS VALUES ('001', 'yourname', 35, 'Extremadura', 5550001)");
  OCI7X_EXEC (htOci7xManager.stSqlInstruction);
  OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto error;);
}
if ( true )
{
  htOci7xManager.stSqlInstruction.format ("INSERT INTO WORKERS VALUES ('001', 'hisname', 25, 'Catalunya', 5550002)");
  OCI7X_EXEC (htOci7xManager.stSqlInstruction);
  OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto error;);
}
if ( true )
{
  htOci7xManager.stSqlInstruction.format ("INSERT INTO WORKERS VALUES ('002', 'john', 35, 'New England', 5550003)");
  OCI7X_EXEC (htOci7xManager.stSqlInstruction);
  OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto error;);
}
if ( true )
{
  htOci7xManager.stSqlInstruction.format ("INSERT INTO WORKERS VALUES ('002', 'james', 35, 'Washington', 5550004)");
  OCI7X_EXEC (htOci7xManager.stSqlInstruction);
  OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto error;);
}
if ( true )
{
  htOci7xManager.stSqlInstruction.format ("INSERT INTO WORKERS SELECT WORK_CODE, NAME, HOURS, STATE, PHONE_NUMBER FROM WORKERS WHERE ( NAME = 'myname' )");
  OCI7X_EXEC (htOci7xManager.stSqlInstruction);
  OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto error;);
}
if ( true )
{
  htOci7xManager.stSqlInstruction.format ("INSERT INTO WORKERS SELECT WORK_CODE, NAME, HOURS, STATE, PHONE_NUMBER FROM WORKERS");
  OCI7X_EXEC (htOci7xManager.stSqlInstruction);
  OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto error;);
}
if ( false )
{
  htOci7xManager.stSqlInstruction.format ("");
  OCI7X_EXEC (htOci7xManager.stSqlInstruction);
  OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto cardinality_check;);
}
{
Cda_Def*   hpsCurrentCda_def;
int        J = 1;

htOci7xManager.initiateEmbeddedVariableMap (NULL , NULL);
htOci7xManager.stSqlInstruction = htOci7xManager.instantiateString ("SELECT HOURS FROM WORKERS, JOBS WHERE ( WORK_CODE = CODE )");
OCI7X_EXEC_CURSOR (htOci7xManager.stSqlInstruction, "OCI7XCN_AUX");
OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto cardinality_check;);
hpsCurrentCda_def = htOci7xManager.cursor ("OCI7XCN_AUX");
if ( !hpsCurrentCda_def )
{
  OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto cardinality_check;);
}
else
{
OCI7X_DEFINE_INPUT (hpsCurrentCda_def, J, stVar1);
OCI7X_UPDATE_STATUS (htOci7xManager.iCode, htOci7xManager.closeCursor ("OCI7XCN_AUX"); goto cardinality_check;);
++J;
  htOci7xManager.iCode = ofetch (hpsCurrentCda_def);
  OCI7X_CHECK_SINGLENESS (hpsCurrentCda_def, htOci7xManager.closeCursor ("OCI7XCN_AUX"); goto cardinality_check;);
  htOci7xManager.closeCursor ("OCI7XCN_AUX"); 
}
}if ( false )
{
  htOci7xManager.stSqlInstruction.format ("SELECT HOURS INTO  FROM WORKERS, JOBS WHERE ( WORK_CODE = CODE )");
  OCI7X_EXEC (htOci7xManager.stSqlInstruction);
  OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto cardinality_check;);
}
}
    TEST_FAIL;


  cardinality_check:

    TEST_STRINGS (SQLSTATE.data(), "21000");
    stAttribName = "WORK_CODE";
    stCursorName = "MYCURSOR";
{
/*
*  OCI 7.x instructions.
*/
if ( false )
{
  htOci7xManager.stSqlInstruction.format ("");
  OCI7X_EXEC (htOci7xManager.stSqlInstruction);
  OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto error;);
}
{
Cda_Def*   hpsCurrentCda_def;
int        J = 1;

htOci7xManager.initiateEmbeddedVariableMap (NULL , NULL);
htOci7xManager.stSqlInstruction = htOci7xManager.instantiateString ("SELECT COUNT (*) FROM WORKERS WHERE ( NAME = 'myname' )");
OCI7X_EXEC_CURSOR (htOci7xManager.stSqlInstruction, "OCI7XCN_AUX");
OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto error;);
hpsCurrentCda_def = htOci7xManager.cursor ("OCI7XCN_AUX");
if ( !hpsCurrentCda_def )
{
  OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto error;);
}
else
{
OCI7X_DEFINE_INPUT (hpsCurrentCda_def, J, stVar2);
OCI7X_UPDATE_STATUS (htOci7xManager.iCode, htOci7xManager.closeCursor ("OCI7XCN_AUX"); goto error;);
++J;
  htOci7xManager.iCode = ofetch (hpsCurrentCda_def);
  OCI7X_CHECK_SINGLENESS (hpsCurrentCda_def, htOci7xManager.closeCursor ("OCI7XCN_AUX"); goto error;);
  htOci7xManager.closeCursor ("OCI7XCN_AUX"); 
}
}if ( false )
{
  htOci7xManager.stSqlInstruction.format ("SELECT COUNT (*) INTO  FROM WORKERS WHERE ( NAME = 'myname' )");
  OCI7X_EXEC (htOci7xManager.stSqlInstruction);
  OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto error;);
}
if ( true )
{
  htOci7xManager.stSqlInstruction.format ("UPDATE WORKERS SET PHONE_NUMBER = WORKERS.PHONE_NUMBER WHERE ( ( WORKERS.NAME = 'myname' ) AND ( WORKERS.NAME = NAME ) )");
  OCI7X_EXEC (htOci7xManager.stSqlInstruction);
  OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto error;);
}
if ( true )
{
  htOci7xManager.stSqlInstruction.format ("REVOKE ALL PRIVILEGES ON JOBS FROM PUBLIC CASCADE CONSTRAINTS");
  OCI7X_EXEC (htOci7xManager.stSqlInstruction);
  OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto error;);
}
if ( true )
{
  htOci7xManager.stSqlInstruction.format ("REVOKE ALL PRIVILEGES ON WORKERS FROM PUBLIC ");
  OCI7X_EXEC (htOci7xManager.stSqlInstruction);
  OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto error;);
}
if ( true )
{
  htOci7xManager.stSqlInstruction.format ("GRANT SELECT ON JOBS TO PUBLIC");
  OCI7X_EXEC (htOci7xManager.stSqlInstruction);
  OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto error;);
}
if ( true )
{
  htOci7xManager.stSqlInstruction.format ("CREATE VIEW VIEW1 AS SELECT * FROM JOBS");
  OCI7X_EXEC (htOci7xManager.stSqlInstruction);
  OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto error;);
}
if ( true )
{
  htOci7xManager.stSqlInstruction.format ("DROP VIEW VIEW1");
  OCI7X_EXEC (htOci7xManager.stSqlInstruction);
  OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto error;);
}
{
TString   hstCursorName;

htOci7xManager.initiateEmbeddedVariableMap (NULL , NULL);
hstCursorName                   = htOci7xManager.instantiateString ("MYCURSOR");
htOci7xManager.stSqlInstruction = htOci7xManager.instantiateString ("SELECT CODE FROM JOBS WHERE ( CODE = '001' )");
OCI7X_DECLARE_CURSOR (htOci7xManager.stSqlInstruction, hstCursorName.data());
OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto error;);
}
if ( false )
{
  htOci7xManager.stSqlInstruction.format ("");
  OCI7X_EXEC (htOci7xManager.stSqlInstruction);
  OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto error;);
}
if ( true )
{
  htOci7xManager.stSqlInstruction.format ("{
  TString   hstCursorName;

  htOci7xManager.initiateEmbeddedVariableMap (NULL , NULL);
  hstCursorName = htOci7xManager.instantiateString ("MYCURSOR");
  OCI7X_OPEN_CURSOR (hstCursorName.data());
  OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto error;);
}
");
  OCI7X_EXEC (htOci7xManager.stSqlInstruction);
  OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto error;);
}
{
TString    hstDynamicCursorName;
Cda_Def*   hpsCurrentCda_def;
int        J = 1;

hstDynamicCursorName.format ("%s", stCursorName.data());
hpsCurrentCda_def = htOci7xManager.cursor (hstDynamicCursorName.data());
if ( !hpsCurrentCda_def )
{
  OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto error;);
}
else
{
  OCI7X_DEFINE_INPUT (hpsCurrentCda_def, J, stVar0);
OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto error;);
++J;
  htOci7xManager.iCode = ofetch (hpsCurrentCda_def);
  OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto error;);
  OCI7X_CHECK_EMPTYNESS (hpsCurrentCda_def, goto not_found;);
}
}
if ( false )
{
  htOci7xManager.stSqlInstruction.format ("", stCursorName.data());
  OCI7X_EXEC (htOci7xManager.stSqlInstruction);
  OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto error;);
}
{
htOci7xManager.initiateEmbeddedVariableMap (NULL , NULL);

htOci7xManager.iCode = htOci7xManager.closeCursor (htOci7xManager.instantiateString ("MYCURSOR"));
OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto error;);
}
if ( false )
{
  htOci7xManager.stSqlInstruction.format ("");
  OCI7X_EXEC (htOci7xManager.stSqlInstruction);
  OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto error;);
}
if ( true )
{
  htOci7xManager.stSqlInstruction.format ("DROP TABLE JOBS CASCADE CONSTRAINTS");
  OCI7X_EXEC (htOci7xManager.stSqlInstruction);
  OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto error;);
}
}

    TEST_STRINGS (SQLSTATE.data(), "00000");
    TEST_STRINGS (stVar2.data(),   "4");
    TEST_STRINGS (stVar0.data(),   "001");

{
/*
*  OCI 7.x instructions.
*/
{
TString   hstCursorName;

htOci7xManager.initiateEmbeddedVariableMap (NULL , stAttribName.data(), NULL);
hstCursorName                   = htOci7xManager.instantiateString ("MYCURSOR");
htOci7xManager.stSqlInstruction = htOci7xManager.instantiateString ("SELECT %s, NAME, HOURS FROM WORKERS");
OCI7X_DECLARE_CURSOR (htOci7xManager.stSqlInstruction, hstCursorName.data());
OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto error;);
}
if ( false )
{
  htOci7xManager.stSqlInstruction.format ("", stAttribName.data());
  OCI7X_EXEC (htOci7xManager.stSqlInstruction);
  OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto error;);
}
if ( true )
{
  htOci7xManager.stSqlInstruction.format ("{
  TString   hstCursorName;

  htOci7xManager.initiateEmbeddedVariableMap (NULL , NULL);
  hstCursorName = htOci7xManager.instantiateString ("MYCURSOR");
  OCI7X_OPEN_CURSOR (hstCursorName.data());
  OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto error;);
}
");
  OCI7X_EXEC (htOci7xManager.stSqlInstruction);
  OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto error;);
}
if ( false )
{
  htOci7xManager.stSqlInstruction.format ("");
  OCI7X_EXEC (htOci7xManager.stSqlInstruction);
  OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto error;);
}
}
    
    if ( _ptTargetFile )
    {
      fprintf (_ptTargetFile, "%s\n", "WORK_CODE NAME                                     HOURS");
      fprintf (_ptTargetFile, "%s\n", "========= ======================================== =====");
    }
    for (;;)
    {
{
/*
*  OCI 7.x instructions.
*/
{
Cda_Def*   hpsCurrentCda_def;
int        J = 1;

htOci7xManager.initiateEmbeddedVariableMap (NULL , NULL);
hpsCurrentCda_def = htOci7xManager.cursor (htOci7xManager.instantiateString ("MYCURSOR"));
if ( !hpsCurrentCda_def )
{
  OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto error;);
}
else
{
  OCI7X_DEFINE_INPUT (hpsCurrentCda_def, J, stVar0);
OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto error;);
++J;
OCI7X_DEFINE_INPUT (hpsCurrentCda_def, J, stVar1);
OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto error;);
++J;
OCI7X_DEFINE_INPUT (hpsCurrentCda_def, J, stVar2);
OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto error;);
++J;
  htOci7xManager.iCode = ofetch (hpsCurrentCda_def);
  OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto error;);
  OCI7X_CHECK_EMPTYNESS (hpsCurrentCda_def, goto loop_end;);
}
}
if ( false )
{
  htOci7xManager.stSqlInstruction.format ("");
  OCI7X_EXEC (htOci7xManager.stSqlInstruction);
  OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto error;);
}
}
      if ( _ptTargetFile )
      {
        fprintf (_ptTargetFile, "%9s %40s %5s\n", stVar0.data(), stVar1.data(), stVar2.data());
      }
      ++zRecordCount;
    }


  loop_end:

{
/*
*  OCI 7.x instructions.
*/
{
htOci7xManager.initiateEmbeddedVariableMap (NULL , NULL);

htOci7xManager.iCode = htOci7xManager.closeCursor (htOci7xManager.instantiateString ("MYCURSOR"));
OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto error;);
}
if ( false )
{
  htOci7xManager.stSqlInstruction.format ("");
  OCI7X_EXEC (htOci7xManager.stSqlInstruction);
  OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto error;);
}
htOci7xManager.iCode = htOci7xManager.disconnectCurrent();
OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto error;);
if ( false )
{
  htOci7xManager.stSqlInstruction.format ("");
  OCI7X_EXEC (htOci7xManager.stSqlInstruction);
  OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto error;);
}
}

{
/*
*  OCI 7.x instructions.
*/
{
TString   hstSqlServerName;
TString   hstConnectionName;
TString   hstUserName;
TString   hstUserId;

htOci7xManager.initiateEmbeddedVariableMap (NULL , stHostName.data(), stUserName.data(), NULL);
hstSqlServerName  = htOci7xManager.instantiateString ("%s");
hstConnectionName = htOci7xManager.instantiateString ("_'myconnection'");
hstUserName       = htOci7xManager.instantiateString ("%s");
hstUserId.format ("%s@%s", hstUserName.data(), hstSqlServerName.data());
hstConnectionName.suppress ('\'');
hstUserId.suppress ('\'');
htOci7xManager.iCode = htOci7xManager.connect (hstConnectionName, (text*) hstUserId.data());
OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto error;);
}
if ( false )
{
  htOci7xManager.stSqlInstruction.format ("", stHostName.data(), stUserName.data());
  OCI7X_EXEC (htOci7xManager.stSqlInstruction);
  OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto error;);
}
if ( true )
{
  htOci7xManager.stSqlInstruction.format ("DROP TABLE WORKERS CASCADE CONSTRAINTS");
  OCI7X_EXEC (htOci7xManager.stSqlInstruction);
  OCI7X_UPDATE_STATUS (htOci7xManager.iCode, goto error;);
}
}
    goto end;

    
  error:

    cerr << "SQLSTATE="    << SQLSTATE << endl;
    cerr << "SQLCODE="     << SQLCODE  << endl;
    cerr << "DESCRIPTION=" << GetErrorMessage() << endl;
    TEST_FAIL_WITH_CODE (SQLCODE);  
    goto end;

    
  not_found:

    cerr << "Search failed\n";
    TEST_FAIL_WITH_CODE (2);
    goto end;
  }

end:
  
  TEST_STOP_WATCH;
  TEST_DISPLAY_WATCH;
  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;

}  /* main() */

