#include "postgresql-libpq-6.2.x.hh"


/*
*  UESQLC::postgresql-libpq-6.2.x template explicit instantiations.
*/

template class TCollection<TString>;
template class TBaseContainer<TString>;
template class TLinkedContainer<TString>;
template class TCollection<PGconn*>;
template class TBaseContainer<PGconn*>;
template class TLinkedContainer<PGconn*>;
template class TBaseMap<TString, PGconn*>;
template class TLinkedMap<TString, PGconn*>;


/*
*  UESQLC::postgresql-libpq-6.2.x internal variables initiation.
*/

TString                 TPostgresql62xManager::stSqlInstruction;
PGconn*                 TPostgresql62xManager::ptPgconn       = NULL;
PGresult*               TPostgresql62xManager::ptPgresult     = NULL;
PGresult*               TPostgresql62xManager::ptAuxPgresult  = NULL;
char*                   TPostgresql62xManager::pcHost         = NULL;
char*                   TPostgresql62xManager::pcDatabaseName = getenv ("USER");


/*
*  C O N S T R U C T O R S
*/

TPostgresql62xManager::~TPostgresql62xManager (void)
{

  finish();

}  /* ~TPostgresql62xManager() */


bool TPostgresql62xManager::initiate (void)
{

  bool   gSuccess = gIsThereConnection;

  if ( !gIsThereConnection )
  {
    ptPgconn = PQsetdb (NULL, NULL, NULL, NULL, getenv ("USER"));
    if ( PQstatus (ptPgconn) == CONNECTION_BAD )
    {
      /*
      *  Condition:           connection exception
      *  Class subcondition:  SQL-client unable to establish SQL-connection
      */
      SQLCODE  = -1;
      SQLSTATE = "08001";
    }
    else
    {
      SQLCODE  = 0;
      SQLSTATE = "00000";
      gSuccess = true;
      connect ("DEFAULT", ptPgconn);
    }
  }
  return gSuccess;

}  /* initiate() */


void TPostgresql62xManager::connect (const TString& rkstCONNECTION_NAME, const PGconn* pktCONNECTION)
{

  tConnectionMap.bind (rkstCONNECTION_NAME, (PGconn*) pktCONNECTION);
  stCurrentConnection = rkstCONNECTION_NAME;
  gIsThereConnection  = true;

}  /* connect() */


void TPostgresql62xManager::disconnect (const TString& rkstCONNECTION_NAME)
{

  PQfinish (tConnectionMap [rkstCONNECTION_NAME]);
  tConnectionMap.unbind (rkstCONNECTION_NAME);
  if ( rkstCONNECTION_NAME == stCurrentConnection )
  {
    stCurrentConnection.clear();
    gIsThereConnection = false;
  }

}  /* disconnect() */


void TPostgresql62xManager::disconnectCurrent (void)
{

  disconnect (stCurrentConnection);

}  /* disconnect() */


PGconn* TPostgresql62xManager::setCurrentConnection (const TString& rkstCONNECTION_NAME)
{

  PGconn*   ptConnectionPgconn = NULL;

  if ( tConnectionMap.isBinded (rkstCONNECTION_NAME) )
  {
    stCurrentConnection = rkstCONNECTION_NAME;
    ptConnectionPgconn  = tConnectionMap [rkstCONNECTION_NAME];
  }
  return ptConnectionPgconn;

}  /* setCurrentConnection() */


/*
*  S E L E C T O R S
*/

bool TPostgresql62xManager::existsDefaultConnection (void) const
{

  return !stCurrentConnection.isEmpty();

}  /* existsDefaultConnection() */


bool TPostgresql62xManager::isConnected (const TString& rkstCONNECTION_NAME) const
{

  return tConnectionMap.isBinded (rkstCONNECTION_NAME);

}  /* isConnected() */


PGconn* TPostgresql62xManager::currentConnection (void) const
{

  PGconn*   ptCurrentPgconn = NULL;

  if ( !stCurrentConnection.isEmpty() )
  {
    ptCurrentPgconn = tConnectionMap [stCurrentConnection];
  }
  return ptCurrentPgconn;

}  /* currentConnection() */


PGconn* TPostgresql62xManager::connection (const TString& rkstCONNECTION_NAME) const
{

  return tConnectionMap [rkstCONNECTION_NAME];

}  /* connection() */


void TPostgresql62xManager::finish (void)
{

  for (register size_t J = 0, zMapCardinal = tConnectionMap.linksCardinal(); ( J < zMapCardinal ) ;J++)
  {
    PQfinish (tConnectionMap.item (J));
  }
  tConnectionMap.clear();
  stCurrentConnection.clear();

}  /* finish() */


