#include "postgresql-libpq-6.2.x.hh"


/*
*  UESQLC::postgresql-libpq-6.2.x internal variables initiation.
*/

TPostgresql62xManager   htPostgresql62xManager;


/*
*  Name:      test-base-1.uesql.cc
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Test for UESQL
*  Date:      $Date: 1998/11/23 11:24:06 $
*  Revision:  $Revision: 1.13 $
*
*  Copyright (C) 1996-1998  Rafael Jesus Alcantara Perez <ralcan@vnet.es>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#include <cstdlib>
#include <iostream>
#include <mpcl/test.h>
#include <mpcl/string.hh>
#include <mpcl/config_processor.hh>


TString GetErrorMessage (void)
{

  TString   stMessage;

  if ( SQLSTATE == "00000" )
  {
    stMessage = "successful completion";
  }
  else if ( SQLSTATE == "02000" )
  {
    stMessage = "no data";
  }
  else if ( SQLSTATE == "08001" )
  {
    stMessage = "connection exception: SQL-client unable to establish SQL-connection";
  }
  else if ( SQLSTATE == "08002" )
  {
    stMessage = "connection exception: connection name in use";
  }
  else if ( SQLSTATE == "08003" )
  {
    stMessage = "connection exception: connection does not exist";
  }
  else if ( SQLSTATE == "08007" )
  {
    stMessage = "connection exception: transaction resolution unknown";
  }
  else if ( SQLSTATE == "21000" )
  {
    stMessage = "cardinality violation";
  }
  else if ( SQLSTATE == "34000" )
  {
    stMessage = "invalid cursor name";
  }
  else if ( SQLSTATE == "40003" )
  {
    stMessage = "transaction rollback: statement completion unknown";
  }
  else if ( SQLSTATE == "42000" )
  {
    stMessage = "syntax error or access rule violation";
  }
  else
  {
    stMessage = "unknown error";
  }
  return stMessage;

}  /* GetErrorMessage() */


/*
*  Parameters and arguments:
*
*    {--hostname, -o}=HOSTNAME
*    {--username, -u}=USERNAME
*    {--password, -p}=PASSWORD
*/
int main (int argc, const char* argv[])
{

  TConfigProcessor   tConfig;
  size_t             zRecordCount = 0;

  TEST_INIT ("test-base-1");
  TEST_START_WATCH;

  TString   stVar0;
  TString   stVar1;
  TString   stVar2;
  TString   stAttribName;
  TString   stCursorName;
  TString   stUserName;
  TString   stHostName;
  TString   stCodeValue = "'002'";

  /*
  *  Define string constants names for options.
  */ 
  tConfig.addOption ("hostname", "o", "localhost");
  tConfig.addOption ("username", "u", getenv ("USER"));
  tConfig.addOption ("password", "p", "");
  tConfig.setCommandLineParameters (argc, argv);
  tConfig.processOptions();
  
  stHostName.format ("'%s'", tConfig ["hostname"].value().data());
  if ( tConfig ["password"].value().isEmpty() )
  {
    stUserName.format ("'%s'", tConfig ["username"].value().data());
  }
  else
  {
    stUserName.format ( "'%s/%s'"                           ,
                        tConfig ["username"].value().data() ,
                        tConfig ["password"].value().data() );
  }

  TEST_WRITE_INFO ("HOSTNAME = %s", stHostName.data());
  TEST_WRITE_INFO ("USER     = %s", stUserName.data());
  if ( !stHostName.isEmpty() && !stUserName.isEmpty() )
  {
{
/*
*  SQL instructions.
*/


{
TString       hstSqlServerDescriptor;
TString       hstSqlServerName;
TString       hstDatabaseName;
TString       hstConnectionName;
const char*   hpakcVariables[] = { NULL , stHostName.data(), stUserName.data() };

hstSqlServerDescriptor.setSeparators ("/");
/*
*  '%s' has been omitted.
*/
hstSqlServerDescriptor = PG62X_INSTANTIATE ("%s",  0, hpakcVariables);
hstConnectionName      = "DEFAULT";
hstSqlServerDescriptor.suppress ('\'');
hstSqlServerName                      = hstSqlServerDescriptor.getToken (0);
hstDatabaseName                       = hstSqlServerDescriptor.getToken (1);
htPostgresql62xManager.pcHost         = hstSqlServerName.data();
htPostgresql62xManager.pcDatabaseName = hstDatabaseName.data();
if ( !htPostgresql62xManager.pcDatabaseName )
{
  htPostgresql62xManager.pcDatabaseName = getenv ("USER");
}
else if ( !strlen (htPostgresql62xManager.pcDatabaseName) )
{
  htPostgresql62xManager.pcDatabaseName = getenv ("USER");
}
htPostgresql62xManager.ptPgconn = PQsetdb (htPostgresql62xManager.pcHost, NULL, NULL, NULL, htPostgresql62xManager.pcDatabaseName);
if ( PQstatus (htPostgresql62xManager.ptPgconn) != CONNECTION_BAD )
{
  SQLCODE  = 0;
  SQLSTATE = "00000";
}
else
{
  /*
  *  Condition:           connection exception
  *  Class subcondition:  SQL-client unable to establish SQL-connection
  */
  SQLCODE  = -1;
  SQLSTATE = "08001";
  goto error;
}
if ( htPostgresql62xManager.isConnected (hstConnectionName) )
{
  /*
  *  Condition:           connection exception
  *  Class subcondition:  connection name in use
  */
  SQLCODE  = -1;
  SQLSTATE = "08002";
  goto error;
}
htPostgresql62xManager.connect (hstConnectionName, htPostgresql62xManager.ptPgconn);
htPostgresql62xManager.setCurrentConnection (hstConnectionName);
}
htPostgresql62xManager.stSqlInstruction.format ("CREATE TABLE JOBS (CODE CHAR (3) NOT NULL, WORKER_COUNT INT4);");
PGV_EXEC (htPostgresql62xManager.stSqlInstruction.data(), htPostgresql62xManager.ptPgconn, goto error;);
htPostgresql62xManager.stSqlInstruction.format ("CREATE TABLE WORKERS (WORK_CODE CHAR (3) NOT NULL, NAME CHAR (40) NOT NULL, HOURS DECIMAL (5));");
PGV_EXEC (htPostgresql62xManager.stSqlInstruction.data(), htPostgresql62xManager.ptPgconn, goto error;);
htPostgresql62xManager.stSqlInstruction.format ("ALTER TABLE WORKERS ADD COLUMN STATE CHAR (40);");
PGV_EXEC (htPostgresql62xManager.stSqlInstruction.data(), htPostgresql62xManager.ptPgconn, goto error;);
htPostgresql62xManager.stSqlInstruction.format ("ALTER TABLE WORKERS ADD COLUMN PHONE_NUMBER INT4;");
PGV_EXEC (htPostgresql62xManager.stSqlInstruction.data(), htPostgresql62xManager.ptPgconn, goto error;);
htPostgresql62xManager.stSqlInstruction.format ("DELETE FROM JOBS WHERE ( CODE = %s );", stCodeValue.data());
PGV_EXEC (htPostgresql62xManager.stSqlInstruction.data(), htPostgresql62xManager.ptPgconn, goto error;);
htPostgresql62xManager.stSqlInstruction.format ("DELETE FROM JOBS WHERE ( CODE = '001' );");
PGV_EXEC (htPostgresql62xManager.stSqlInstruction.data(), htPostgresql62xManager.ptPgconn, goto error;);
htPostgresql62xManager.stSqlInstruction.format ("DELETE FROM WORKERS;");
PGV_EXEC ("DELETE FROM WORKERS;", htPostgresql62xManager.ptPgconn, goto error;);
htPostgresql62xManager.stSqlInstruction.format ("INSERT INTO JOBS VALUES ('001', 100);");
PG_EXEC (htPostgresql62xManager.stSqlInstruction.data(), htPostgresql62xManager.ptPgconn, goto error;);
htPostgresql62xManager.stSqlInstruction.format ("INSERT INTO JOBS VALUES ('002', 250);");
PG_EXEC (htPostgresql62xManager.stSqlInstruction.data(), htPostgresql62xManager.ptPgconn, goto error;);
htPostgresql62xManager.stSqlInstruction.format ("INSERT INTO WORKERS VALUES ('001', 'myname', 40, 'Andalucia', 5550000);");
PG_EXEC (htPostgresql62xManager.stSqlInstruction.data(), htPostgresql62xManager.ptPgconn, goto error;);
htPostgresql62xManager.stSqlInstruction.format ("INSERT INTO WORKERS VALUES ('001', 'yourname', 35, 'Extremadura', 5550001);");
PG_EXEC (htPostgresql62xManager.stSqlInstruction.data(), htPostgresql62xManager.ptPgconn, goto error;);
htPostgresql62xManager.stSqlInstruction.format ("INSERT INTO WORKERS VALUES ('001', 'hisname', 25, 'Catalunya', 5550002);");
PG_EXEC (htPostgresql62xManager.stSqlInstruction.data(), htPostgresql62xManager.ptPgconn, goto error;);
htPostgresql62xManager.stSqlInstruction.format ("INSERT INTO WORKERS VALUES ('002', 'john', 35, 'New England', 5550003);");
PG_EXEC (htPostgresql62xManager.stSqlInstruction.data(), htPostgresql62xManager.ptPgconn, goto error;);
htPostgresql62xManager.stSqlInstruction.format ("INSERT INTO WORKERS VALUES ('002', 'james', 35, 'Washington', 5550004);");
PG_EXEC (htPostgresql62xManager.stSqlInstruction.data(), htPostgresql62xManager.ptPgconn, goto error;);
htPostgresql62xManager.stSqlInstruction.format ("INSERT INTO WORKERS SELECT WORK_CODE, NAME, HOURS, STATE, PHONE_NUMBER FROM WORKERS WHERE ( NAME = 'myname' );");
PG_EXEC (htPostgresql62xManager.stSqlInstruction.data(), htPostgresql62xManager.ptPgconn, goto error;);
htPostgresql62xManager.stSqlInstruction.format ("INSERT INTO WORKERS SELECT WORK_CODE, NAME, HOURS, STATE, PHONE_NUMBER FROM WORKERS;");
PG_EXEC (htPostgresql62xManager.stSqlInstruction.data(), htPostgresql62xManager.ptPgconn, goto error;);

{
int   J = 0;

PGV_EXEC ("BEGIN;", htPostgresql62xManager.ptPgconn, goto cardinality_check;);
htPostgresql62xManager.stSqlInstruction.format ("DECLARE AUX_CURSOR CURSOR FOR SELECT HOURS FROM WORKERS, JOBS WHERE ( WORK_CODE = CODE );");
PGV_EXEC (htPostgresql62xManager.stSqlInstruction.data(), htPostgresql62xManager.ptPgconn, goto cardinality_check;);
PG_EXEC ("FETCH ALL IN AUX_CURSOR;", htPostgresql62xManager.ptPgconn, goto cardinality_check;);
if ( PG62X_NOT_FOUND_CONDITION )
{
  /* 
  *  Condition:  no data
  */ 
  SQLCODE = 101; SQLSTATE = "02000";
  if ( htPostgresql62xManager.ptPgresult ) PQclear (htPostgresql62xManager.ptPgresult);
  PGIV_EXEC ("CLOSE AUX_CURSOR;", htPostgresql62xManager.ptPgconn);
  PGIV_EXEC ("END;", htPostgresql62xManager.ptPgconn);
  goto not_found;
}
else if ( PQntuples (htPostgresql62xManager.ptPgresult) != 1 )
{
  /* 
  *  Condition:  cardinality violation
  */ 
  SQLCODE = -1; SQLSTATE = "21000"; 
  if ( htPostgresql62xManager.ptPgresult ) PQclear (htPostgresql62xManager.ptPgresult);
  PGIV_EXEC ("CLOSE AUX_CURSOR;", htPostgresql62xManager.ptPgconn);
  PGIV_EXEC ("END;", htPostgresql62xManager.ptPgconn);
  goto cardinality_check;
}
else
{
  SQLCODE = 0; SQLSTATE = "00000";
  stVar1 = PQgetvalue (htPostgresql62xManager.ptPgresult, 0, J++);
  if ( htPostgresql62xManager.ptPgresult ) PQclear (htPostgresql62xManager.ptPgresult);
  PGV_EXEC ("CLOSE AUX_CURSOR;", htPostgresql62xManager.ptPgconn, goto cardinality_check;);
  PGV_EXEC ("END;", htPostgresql62xManager.ptPgconn, goto cardinality_check;);
}
}

}
    TEST_FAIL;


  cardinality_check:

    TEST_STRINGS (SQLSTATE.data(), "21000");
    stAttribName = "WORK_CODE";
    stCursorName = "MYCURSOR";
{
/*
*  SQL instructions.
*/

{
int   J = 0;

PGV_EXEC ("BEGIN;", htPostgresql62xManager.ptPgconn, goto error;);
htPostgresql62xManager.stSqlInstruction.format ("DECLARE AUX_CURSOR CURSOR FOR SELECT COUNT (*) FROM WORKERS WHERE ( NAME = 'myname' );");
PGV_EXEC (htPostgresql62xManager.stSqlInstruction.data(), htPostgresql62xManager.ptPgconn, goto error;);
PG_EXEC ("FETCH ALL IN AUX_CURSOR;", htPostgresql62xManager.ptPgconn, goto error;);
if ( PG62X_NOT_FOUND_CONDITION )
{
  /* 
  *  Condition:  no data
  */ 
  SQLCODE = 101; SQLSTATE = "02000";
  if ( htPostgresql62xManager.ptPgresult ) PQclear (htPostgresql62xManager.ptPgresult);
  PGIV_EXEC ("CLOSE AUX_CURSOR;", htPostgresql62xManager.ptPgconn);
  PGIV_EXEC ("END;", htPostgresql62xManager.ptPgconn);
  goto not_found;
}
else if ( PQntuples (htPostgresql62xManager.ptPgresult) != 1 )
{
  /* 
  *  Condition:  cardinality violation
  */ 
  SQLCODE = -1; SQLSTATE = "21000"; 
  if ( htPostgresql62xManager.ptPgresult ) PQclear (htPostgresql62xManager.ptPgresult);
  PGIV_EXEC ("CLOSE AUX_CURSOR;", htPostgresql62xManager.ptPgconn);
  PGIV_EXEC ("END;", htPostgresql62xManager.ptPgconn);
  goto error;
}
else
{
  SQLCODE = 0; SQLSTATE = "00000";
  stVar2 = PQgetvalue (htPostgresql62xManager.ptPgresult, 0, J++);
  if ( htPostgresql62xManager.ptPgresult ) PQclear (htPostgresql62xManager.ptPgresult);
  PGV_EXEC ("CLOSE AUX_CURSOR;", htPostgresql62xManager.ptPgconn, goto error;);
  PGV_EXEC ("END;", htPostgresql62xManager.ptPgconn, goto error;);
}
}
htPostgresql62xManager.stSqlInstruction.format ("UPDATE WORKERS SET PHONE_NUMBER = WORKERS.PHONE_NUMBER WHERE ( ( WORKERS.NAME = 'myname' ) AND ( WORKERS.NAME = NAME ) );");
PG_EXEC (htPostgresql62xManager.stSqlInstruction.data(), htPostgresql62xManager.ptPgconn, goto error;);
/*
*  'CASCADE' has been omitted.
*/
htPostgresql62xManager.stSqlInstruction.format ("REVOKE ALL PRIVILEGES ON JOBS FROM PUBLIC;");
PGV_EXEC (htPostgresql62xManager.stSqlInstruction.data(), htPostgresql62xManager.ptPgconn, goto error;);
/*
*  'RESTRICT' has been omitted.
*/
htPostgresql62xManager.stSqlInstruction.format ("REVOKE ALL PRIVILEGES ON WORKERS FROM PUBLIC;");
PGV_EXEC (htPostgresql62xManager.stSqlInstruction.data(), htPostgresql62xManager.ptPgconn, goto error;);
htPostgresql62xManager.stSqlInstruction.format ("GRANT SELECT ON JOBS TO PUBLIC;");
PGV_EXEC (htPostgresql62xManager.stSqlInstruction.data(), htPostgresql62xManager.ptPgconn, goto error;);
htPostgresql62xManager.stSqlInstruction.format ("CREATE VIEW VIEW1 AS SELECT * FROM JOBS;");
PGV_EXEC (htPostgresql62xManager.stSqlInstruction.data(), htPostgresql62xManager.ptPgconn, goto error;);
/*
*  'CASCADE' has been omitted.
*/
htPostgresql62xManager.stSqlInstruction.format ("DROP VIEW VIEW1;");
PGV_EXEC (htPostgresql62xManager.stSqlInstruction.data(), htPostgresql62xManager.ptPgconn, goto error;);
PGV_EXEC ("BEGIN;", htPostgresql62xManager.ptPgconn, goto error;);
htPostgresql62xManager.stSqlInstruction.format ("DECLARE MYCURSOR CURSOR FOR SELECT CODE FROM JOBS WHERE ( CODE = '001' );");
PGV_EXEC (htPostgresql62xManager.stSqlInstruction.data(), htPostgresql62xManager.ptPgconn, goto error;);

{
int   J = 0;

htPostgresql62xManager.stSqlInstruction.format ("FETCH IN %s;", stCursorName.data());
PG_EXEC (htPostgresql62xManager.stSqlInstruction.data(), htPostgresql62xManager.ptPgconn, goto error;);
if ( PG62X_NOT_FOUND_CONDITION )
{
  /* 
  *  Condition:  no data
  */ 
  SQLCODE = 101; SQLSTATE = "02000";
  if ( htPostgresql62xManager.ptPgresult ) { PQclear (htPostgresql62xManager.ptPgresult); }
  goto not_found;
}
else
{
  stVar0 = PQgetvalue (htPostgresql62xManager.ptPgresult, 0, J++);
  if ( htPostgresql62xManager.ptPgresult ) { PQclear (htPostgresql62xManager.ptPgresult); }
}
}
htPostgresql62xManager.stSqlInstruction.format ("CLOSE MYCURSOR;");
PGV_EXEC (htPostgresql62xManager.stSqlInstruction.data(), htPostgresql62xManager.ptPgconn, goto error;);
PGV_EXEC ("END;", htPostgresql62xManager.ptPgconn, goto error;);

/*
*  'CASCADE' has been omitted.
*/
htPostgresql62xManager.stSqlInstruction.format ("DROP TABLE JOBS;");
PGV_EXEC (htPostgresql62xManager.stSqlInstruction.data(), htPostgresql62xManager.ptPgconn, goto error;);

}

    TEST_STRINGS (SQLSTATE.data(), "00000");
    TEST_STRINGS (stVar2.data(),   "4");
    TEST_STRINGS (stVar0.data(),   "001");

{
/*
*  SQL instructions.
*/
PGV_EXEC ("BEGIN;", htPostgresql62xManager.ptPgconn, goto error;);
htPostgresql62xManager.stSqlInstruction.format ("DECLARE MYCURSOR CURSOR FOR SELECT %s, NAME, HOURS FROM WORKERS;", stAttribName.data());
PGV_EXEC (htPostgresql62xManager.stSqlInstruction.data(), htPostgresql62xManager.ptPgconn, goto error;);



}
    
    if ( _ptTargetFile )
    {
      fprintf (_ptTargetFile, "%s\n", "WORK_CODE NAME                                     HOURS");
      fprintf (_ptTargetFile, "%s\n", "========= ======================================== =====");
    }
    for (;;)
    {
{
/*
*  SQL instructions.
*/
{
int   J = 0;

htPostgresql62xManager.stSqlInstruction.format ("FETCH IN MYCURSOR;");
PG_EXEC (htPostgresql62xManager.stSqlInstruction.data(), htPostgresql62xManager.ptPgconn, goto error;);
if ( PG62X_NOT_FOUND_CONDITION )
{
  /* 
  *  Condition:  no data
  */ 
  SQLCODE = 101; SQLSTATE = "02000";
  if ( htPostgresql62xManager.ptPgresult ) { PQclear (htPostgresql62xManager.ptPgresult); }
  goto loop_end;
}
else
{
  stVar0 = PQgetvalue (htPostgresql62xManager.ptPgresult, 0, J++);
stVar1 = PQgetvalue (htPostgresql62xManager.ptPgresult, 0, J++);
stVar2 = PQgetvalue (htPostgresql62xManager.ptPgresult, 0, J++);
  if ( htPostgresql62xManager.ptPgresult ) { PQclear (htPostgresql62xManager.ptPgresult); }
}
}

}
      if ( _ptTargetFile )
      {
        fprintf (_ptTargetFile, "%9s %40s %5s\n", stVar0.data(), stVar1.data(), stVar2.data());
      }
      ++zRecordCount;
    }


  loop_end:

{
/*
*  SQL instructions.
*/
htPostgresql62xManager.stSqlInstruction.format ("CLOSE MYCURSOR;");
PGV_EXEC (htPostgresql62xManager.stSqlInstruction.data(), htPostgresql62xManager.ptPgconn, goto error;);
PGV_EXEC ("END;", htPostgresql62xManager.ptPgconn, goto error;);

{ if ( htPostgresql62xManager.existsDefaultConnection() )
{
  SQLCODE  = 0;
  SQLSTATE = "00000";
  htPostgresql62xManager.disconnectCurrent();
}
else
{
  /*
  *  Condition:           connection exception
  *  Class subcondition:  connection does not exist
  */
  SQLCODE  = -1;
  SQLSTATE = "08003";
  goto error;
}
 }

}

{
/*
*  SQL instructions.
*/
{
TString       hstSqlServerDescriptor;
TString       hstSqlServerName;
TString       hstDatabaseName;
TString       hstConnectionName;
const char*   hpakcVariables[] = { NULL , stHostName.data(), stUserName.data() };

hstSqlServerDescriptor.setSeparators ("/");
/*
*  '%s' has been omitted.
*/
hstSqlServerDescriptor = PG62X_INSTANTIATE ("%s",  0, hpakcVariables);
hstConnectionName      = PG62X_INSTANTIATE ("_'myconnection'", 1, hpakcVariables);
hstSqlServerDescriptor.suppress ('\'');
hstConnectionName.suppress ('\'');
hstSqlServerName                      = hstSqlServerDescriptor.getToken (0);
hstDatabaseName                       = hstSqlServerDescriptor.getToken (1);
htPostgresql62xManager.pcHost         = hstSqlServerName.data();
htPostgresql62xManager.pcDatabaseName = hstDatabaseName.data();
if ( !htPostgresql62xManager.pcDatabaseName )
{
  htPostgresql62xManager.pcDatabaseName = getenv ("USER");
}
else if ( !strlen (htPostgresql62xManager.pcDatabaseName) )
{
  htPostgresql62xManager.pcDatabaseName = getenv ("USER");
}
htPostgresql62xManager.ptPgconn = PQsetdb (htPostgresql62xManager.pcHost, NULL, NULL, NULL, htPostgresql62xManager.pcDatabaseName);
if ( PQstatus (htPostgresql62xManager.ptPgconn) != CONNECTION_BAD )
{
  SQLCODE  = 0;
  SQLSTATE = "00000";
}
else
{
  /*
  *  Condition:           connection exception
  *  Class subcondition:  SQL-client unable to establish SQL-connection
  */
  SQLCODE  = -1;
  SQLSTATE = "08001";
  goto error;
}
if ( htPostgresql62xManager.isConnected (hstConnectionName) )
{
  /*
  *  Condition:           connection exception
  *  Class subcondition:  connection name in use
  */
  SQLCODE  = -1;
  SQLSTATE = "08002";
  goto error;
}
htPostgresql62xManager.connect (hstConnectionName, htPostgresql62xManager.ptPgconn);
htPostgresql62xManager.setCurrentConnection (hstConnectionName);
}
/*
*  'CASCADE' has been omitted.
*/
htPostgresql62xManager.stSqlInstruction.format ("DROP TABLE WORKERS;");
PGV_EXEC (htPostgresql62xManager.stSqlInstruction.data(), htPostgresql62xManager.ptPgconn, goto error;);

}
    goto end;

    
  error:

    cerr << "SQLSTATE="    << SQLSTATE << endl;
    cerr << "SQLCODE="     << SQLCODE  << endl;
    cerr << "DESCRIPTION=" << GetErrorMessage() << endl;
    TEST_FAIL_WITH_CODE (SQLCODE);  
    goto end;

    
  not_found:

    cerr << "Search failed\n";
    TEST_FAIL_WITH_CODE (2);
    goto end;
  }

end:
  
  TEST_STOP_WATCH;
  TEST_DISPLAY_WATCH;
  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;

}  /* main() */

